@@ -2,6 +2,7 @@
 # alarm.tcl v 1.0	2002/07/21   BurgerMan
 #########################################################
 
+::Version::setSubversionId {$Id: alarm.tcl 9337 2008-01-10 19:24:20Z tjikkun $}
 
 #::Alarms namespace. Everything related to alarms (alerts)
 namespace eval ::alarms {
@@ -80,6 +81,11 @@ namespace eval ::alarms {
 	
 		#Create window if not "embedded" mode
 		if { $window == "" } {
+			if { $user == "all" } {
+				set windowtitle [trans alarmpref2]
+			} else {
+				set windowtitle "[trans alarmpref] $user"
+			}
 			set w ".alarm_cfg_[::md5::md5 $user]"
 			if { [ winfo exists $w ] } {
 				catch { raise $w }
@@ -87,7 +93,7 @@ namespace eval ::alarms {
 				return
 			}
 			toplevel $w
-			wm title $w "[trans alarmpref] $user"
+			wm title $w $windowtitle
 			wm iconname $w [trans alarmpref]
 		} else {
 			set w $window
@@ -97,7 +103,7 @@ namespace eval ::alarms {
 	
 		#If window mode, set a title
 		if { $window == "" } {
-			label $w.title -text "[trans alarmpref]: $user" -font bboldf
+			label $w.title -text $windowtitle -font bboldf
 			pack $w.title -side top -padx 15 -pady 15
 		}
 		
@@ -207,7 +213,7 @@ namespace eval ::alarms {
 		::abook::saveToDisk
 		InitMyAlarms $user
 		
-		cmsn_draw_online 0 2
+		::Event::fireEvent contactAlarmChange gui $user
 	}
 
 	#Saves alarm settings for current user on OK press.
@@ -278,7 +284,7 @@ namespace eval ::alarms {
 		::abook::setContactData $user alarms [array get alarms]
 		::abook::saveToDisk
 		
-		cmsn_draw_online 0 2
+		::Event::fireEvent contactAlarmChange gui $user
 		unset my_alarms
 		
 		return 0
@@ -289,13 +295,26 @@ namespace eval ::alarms {
 		$snd stop
 		$snd destroy
 	}
+
+	proc contactChanged { eventused user } {
+		if { $eventused == "contactStateChange" } {
+			set custom_user_name [::abook::getDisplayNick $user]
+			set status "[trans [::MSN::stateToDescription [::abook::getVolatileData $user state]]]"
+			if { ( [::alarms::isEnabled $user] == 1 )&& ( [::alarms::getAlarmItem $user onstatus] == 1) } {
+				run_alarm $user $user $custom_user_name "[trans changestate $custom_user_name $status]"
+			} elseif { ( [::alarms::isEnabled all] == 1 )&& ( [::alarms::getAlarmItem all onstatus] == 1)} {
+				run_alarm all $user $custom_user_name "[trans changestate $custom_user_name $status]"
+			}
+		}
+	}
+	::Event::registerEvent contactStateChange all ::alarms::contactChanged
 }
 
 
 
 #Runs the alarm (sound and pic)
 proc run_alarm {config_user user nick msg} {
-	global program_dir tcl_platform alarm_win_number
+	global program_dir alarm_win_number
 	
 	if { ![info exists alarm_win_number] } {
 		set alarm_win_number 0
@@ -306,9 +325,9 @@ proc run_alarm {config_user user nick msg} {
 
 	if { [::alarms::getAlarmItem ${config_user} pic_st] == 1 || [::alarms::getAlarmItem ${config_user} sound_st] == 1 } {
 		toplevel .${wind_name}
-		set myDate [ clock format [clock seconds] -format " - %d/%m/%y at %H:%M" ]
-		wm title .${wind_name} "[trans alarm] $user $myDate"	
-		label .${wind_name}.txt -text "$msg"
+		set myDate [ clock format [clock seconds] -format "%d %b %Y %T" ]
+		wm title .${wind_name} "[trans alarm] - $nick"
+		label .${wind_name}.txt -text "$nick ($user)\n[trans on] $myDate\n\n$msg\n"
 		pack .${wind_name}.txt
 	}
 	
@@ -391,7 +410,7 @@ proc run_alarm {config_user user nick msg} {
 }
 
 # Switches alarm setting from ON/OFF
-proc switch_alarm { user tw icon} {
+proc switch_alarm { user } {
 	#We get the alarms configuration. It's stored as a list, but it's converted to an array
 	array set alarms [::abook::getContactData $user alarms]
 
@@ -403,16 +422,5 @@ proc switch_alarm { user tw icon} {
 	
 	#We set the alarms configuration. We can't store an array, so we convert it to a list
 	::abook::setContactData $user alarms [array get alarms]
-	redraw_alarm_icon $user $tw $icon
-}
-
-# Redraws the alarm icon for current user ONLY without redrawing full list of contacts
-proc redraw_alarm_icon { user tw icon } {
-
-	if { [::alarms::getAlarmItem $user enabled] == 1 } {
-		$tw image configure $icon -image [::skin::loadPixmap bell]
-	} else {
-		$tw image configure $icon -image [::skin::loadPixmap belloff]
-	}
 }
 
