@@ -1,4 +1,7 @@
-package require AMSN_BWidget
+
+::Version::setSubversionId {$Id: autoupdate.tcl 11886 2010-01-11 01:43:13Z kakaroto $}
+
+package require BWidget
 
 namespace eval ::autoupdate {
 
@@ -6,29 +9,32 @@ namespace eval ::autoupdate {
 		global tcl_platform
 		global tlsplatform
 
-		if { ($tcl_platform(os) == "Linux") && ($tcl_platform(machine) == "ppc")} {
-			set tlsplatform "linuxppc"
-		} elseif { ($tcl_platform(os) == "Linux") && ($tcl_platform(machine) == "sparc")} {
-			set tlsplatform "linuxsparc"
-		} elseif { ($tcl_platform(os) == "Linux") && ($tcl_platform(machine) == "x86_64")} {
-			set tlsplatform "linuxx86_64"
-		} elseif { $tcl_platform(os) == "Linux" } {
-			set tlsplatform "linuxx86"
+		if { [OnLinux] } {
+			if { ($tcl_platform(machine) == "ppc")} {
+				set tlsplatform "linuxppc"
+			} elseif { ($tcl_platform(machine) == "sparc")} {
+				set tlsplatform "linuxsparc"
+			} elseif { ($tcl_platform(machine) == "x86_64")} {
+				set tlsplatform "linuxx86_64"
+			} else {
+				set tlsplatform "linuxx86"
+			}
 		} elseif { $tcl_platform(os) == "NetBSD"} {
 			set tlsplatform "netbsdx86"
 		} elseif { $tcl_platform(os) == "FreeBSD"} {
 			set tlsplatform "freebsdx86"
 		} elseif { $tcl_platform(os) == "Solaris"} {
 			set tlsplatform "solaris28"
-		} elseif { $tcl_platform(platform) == "windows"} {
+		} elseif { [OnWin] } {
 			set tlsplatform "win32"
-		} elseif { $tcl_platform(os) == "Darwin" } {
+		} elseif { [OnDarwin] } {
 			set tlsplatform "mac"
 		} else {
 			set tlsplatform "src"
 		}
 
 		toplevel .tlsdown
+		wm group .tlsdown .
 		wm title .tlsdown "[trans tlsinstall]"
 
 		label .tlsdown.caption -text "[trans tlsinstallexp]" -anchor w -font splainf
@@ -38,20 +44,13 @@ namespace eval ::autoupdate {
 		pack .tlsdown.caption -side top -expand true -fill x -padx 10 -pady 10
 		pack .tlsdown.choose -side top -anchor w -padx 10 -pady 10
 
-		radiobutton .tlsdown.linuxx86 -text "Linux-x86" -variable tlsplatform -value "linuxx86" -font splainf
-		radiobutton .tlsdown.linuxx86_64 -text "Linux-x86_64" -variable tlsplatform -value "linuxx86_64" -font splainf
-		radiobutton .tlsdown.linuxppc -text "Linux-PowerPC" -variable tlsplatform -value "linuxppc" -font splainf
-		radiobutton .tlsdown.linuxsparc -text "Linux-SPARC" -variable tlsplatform -value "linuxsparc" -font splainf
-		radiobutton .tlsdown.netbsdx86 -text "NetBSD-x86" -variable tlsplatform -value "netbsdx86" -font splainf
-		radiobutton .tlsdown.netbsdsparc64 -text "NetBSD-SPARC64" -variable tlsplatform -value "netbsdsparc64" -font splainf
-		radiobutton .tlsdown.freebsdx86 -text "FreeBSD-x86" -variable tlsplatform -value "freebsdx86" -font splainf
-		radiobutton .tlsdown.solaris26 -text "Solaris 2.6 SPARC" -variable tlsplatform -value "solaris26" -font splainf
-		radiobutton .tlsdown.solaris28 -text "Solaris 2.8 SPARC" -variable tlsplatform -value "solaris28" -font splainf
-		radiobutton .tlsdown.win32 -text "Windows" -variable tlsplatform -value "win32" -font splainf
-		radiobutton .tlsdown.mac -text "Mac OS X" -variable tlsplatform -value "mac" -font splainf
-		radiobutton .tlsdown.src -text "[trans sourcecode]" -variable tlsplatform -value "src" -font splainf
-		radiobutton .tlsdown.nodown -text "[trans dontdownload]" -variable tlsplatform -value "nodown" -font splainf
-
+		combobox::combobox .tlsdown.os -editable false -width 60 -command ::autoupdate::TLS_OS_choosed
+		.tlsdown.os list delete 0 end
+		set trans_sourcecode [trans sourcecode]
+		set trans_dontdownload [trans dontdownload]
+		set list_os_name [list Linux-x86 Linux-x86_64 Linux-PowerPC Linux-SPARC NetBSD-x86 NetBSD-SPARC64 FreeBSD-x86 "Solaris 2.6 SPARC" "Solaris 2.8 SPARC" Windows "Mac OS X" $trans_sourcecode $trans_dontdownload ]
+		eval .tlsdown.os list insert end $list_os_name
+		.tlsdown.os select 0
 
 		frame .tlsdown.f
 
@@ -61,19 +60,7 @@ namespace eval ::autoupdate {
 		pack .tlsdown.f.cancel -side right -padx 10 -pady 10
 		pack .tlsdown.f.ok -side right -padx 10 -pady 10
 
-		pack .tlsdown.linuxx86 -side top -anchor w -padx 15
-		pack .tlsdown.linuxx86_64 -side top -anchor w -padx 15
-		pack .tlsdown.linuxppc -side top -anchor w -padx 15
-		pack .tlsdown.linuxsparc -side top -anchor w -padx 15
-		pack .tlsdown.netbsdx86 -side top -anchor w -padx 15
-		pack .tlsdown.netbsdsparc64 -side top -anchor w -padx 15
-		pack .tlsdown.freebsdx86 -side top -anchor w -padx 15
-		pack .tlsdown.solaris26 -side top -anchor w -padx 15
-		pack .tlsdown.solaris28 -side top -anchor w -padx 15
-		pack .tlsdown.mac -side top -anchor w -padx 15
-		pack .tlsdown.win32 -side top -anchor w -padx 15
-		pack .tlsdown.src -side top -anchor w -padx 15
-		pack .tlsdown.nodown -side top -anchor w -padx 15
+		pack .tlsdown.os
 
 		pack .tlsdown.f -side top
 
@@ -86,6 +73,21 @@ namespace eval ::autoupdate {
 		catch {grab .tlsdown}
 	}
 
+	proc TLS_OS_choosed {w value} {
+		global tlsplatform
+		set trans_sourcecode [trans sourcecode]
+		set trans_dontdownload [trans dontdownload]
+		set list_os_name [list Linux-x86 Linux-x86_64 Linux-PowerPC Linux-SPARC NetBSD-x86 NetBSD-SPARC64 FreeBSD-x86 "Solaris 2.6 SPARC" "Solaris 2.8 SPARC" Windows "Mac OS X" $trans_sourcecode $trans_dontdownload ]
+		set list_os_value [list linuxx86 linuxx86_64 linuxppc linuxsparc netbsdx86 netbsdsparc64 freebsdx86 solaris26 solaris28 win32 mac src nodown]
+		set t 0
+		foreach x $list_os_name {
+			if {$value eq $x} {
+				set tlsplatform [lindex $list_os_value $t]
+			}
+			incr t
+		}
+	}
+
 	proc downloadTLS {} {
 		global tlsplatform
 
@@ -94,11 +96,11 @@ namespace eval ::autoupdate {
 			return
 		}
 
-		set downloadurl "http://www.amsn-project.net/download-tls.php?arch=$tlsplatform"
+		set downloadurl "$::weburl/download-tls.php?arch=$tlsplatform"
 
 
 		if {[ catch {set tok [::http::geturl $downloadurl -command "::autoupdate::downloadTLS2 $downloadurl"]} res ]} {
-			::autoupdate::errorDownloadingTLS $res
+			::autoupdate::errorDownloadingTLS "" $res
 			catch {::http::cleanup $tok}
 		}
 	}
@@ -106,12 +108,11 @@ namespace eval ::autoupdate {
 	proc downloadTLS2 { downloadurl token } {
 		status_log "status: [http::status $token]\n"
 		if { [::http::ncode $token] == 302} {
-
-
 			upvar #0 $token state
 			array set meta $state(meta)
 			set downloadurl $meta(Location)
 			status_log "got referal to $downloadurl"
+			catch {::http::cleanup $token}
 
 			set w .tlsprogress
 
@@ -133,23 +134,27 @@ namespace eval ::autoupdate {
 
 			wm title $w "[trans tlsinstall]"
 
-			::dkfprogress::SetProgress $w.prbar 0
+			::autoupdate::downloadTLSRedirect $w $downloadurl 
 
-			if {[ catch {set tok [::http::geturl $downloadurl -progress "::autoupdate::downloadTLSProgress $downloadurl" -command "::autoupdate::downloadTLSCompleted $downloadurl"]} res ]} {
-				::autoupdate::errorDownloadingTLS $res
-				catch {::http::cleanup $tok}
-			} else {
-				$w.close configure -command "::http::reset $tok"
-				wm protocol $w WM_DELETE_WINDOW "::http::reset $tok"
-			}
 		} else {
-			::autoupdate::errorDownloadingTLS "No redirect from $downloadurl"
+			::autoupdate::errorDownloadingTLS "" [trans cantget $downloadurl]
 			catch {::http::cleanup $token}
 		}
 	}
+	proc downloadTLSRedirect { w downloadurl } {
+		$w.l configure -text "[trans downloadingtls $downloadurl]" 
+		$w.progress configure -text "[trans receivedbytes 0 ?]" 
+		::dkfprogress::SetProgress $w.prbar 0
 		
-
-	proc downloadTLSCompleted { downloadurl token } {
+		if {[ catch {set tok [::http::geturl $downloadurl -progress "::autoupdate::downloadTLSProgress $w $downloadurl" -command "::autoupdate::downloadTLSCompleted $w $downloadurl"]} res ]} {
+			::autoupdate::errorDownloadingTLS $w $res
+			catch {::http::cleanup $tok}
+		} else {
+			$w.close configure -command "::http::reset $tok"
+			wm protocol $w WM_DELETE_WINDOW "::http::reset $tok"
+		}
+	}
+	proc downloadTLSCompleted { w downloadurl token } {
 		global HOME2 tlsplatform
 		status_log "status: [http::status $token]\n"
 		if { [::http::status $token] == "reset" } {
@@ -159,8 +164,21 @@ namespace eval ::autoupdate {
 			return
 		}
 
-		if { [::http::status $token] != "ok" || [::http::ncode $token] != 200} {
-			::autoupdate::errorDownloadingTLS "Couldn't get $downloadurl"
+		if { [::http::status $token] == "ok" && ([::http::ncode $token] == 301 || [::http::ncode $token] == 302)} {
+			upvar #0 $token state
+			
+			array set meta $state(meta)
+			if {[info exists meta(Location)] } {
+				after 100 [list ::autoupdate::downloadTLSRedirect $w $meta(Location)]
+			} else {
+				::autoupdate::errorDownloadingTLS $w [trans cantget $downloadurl]
+			}
+
+			::http::cleanup $token
+			return
+		} elseif { [::http::status $token] != "ok" || [::http::ncode $token] != 200} {
+			::autoupdate::errorDownloadingTLS $w  [trans cantget $downloadurl]
+			::http::cleanup $token
 			return
 		}
 
@@ -168,6 +186,11 @@ namespace eval ::autoupdate {
 		set fname [string range $downloadurl $lastslash end]
 
 
+		if { [string length [::http::data $token]] == 0 } {
+			::autoupdate::errorDownloadingTLS $w [trans cantget $downloadurl]
+			return
+		}
+
 		switch $tlsplatform {
 			"solaris26" -
 			"solaris28" -
@@ -180,13 +203,13 @@ namespace eval ::autoupdate {
 			"freebsdx86" -
 			"mac" -
 			"win32" {
+				set olddir [pwd]
 				if { [catch {
 					set file_id [open [file join [::config::getKey receiveddir] $fname] w]
 					fconfigure $file_id -translation {binary binary} -encoding binary
 					puts -nonewline $file_id [::http::data $token]
 					close $file_id
 
-					set olddir [pwd]
 					cd [file join $HOME2 plugins]
 
 					if { $tlsplatform == "win32" } {
@@ -197,8 +220,9 @@ namespace eval ::autoupdate {
 					cd $olddir
 					::amsn::infoMsg "[trans tlsinstcompleted]"
 				} res ] } {
+					cd $olddir
 
-					::autoupdate::errorDownloadingTLS $res
+					::autoupdate::errorDownloadingTLS $w $res
 				}
 			}
 			"src" {
@@ -209,7 +233,7 @@ namespace eval ::autoupdate {
 					close $file_id
 					::amsn::infoMsg "[trans tlsdowncompleted $fname [::config::getKey receiveddir] [file join $HOME2 plugins]]"
 				} res ] } {
-					::autoupdate::errorDownloadingTLS $res
+					::autoupdate::errorDownloadingTLS $w $res
 				}
 			}
 			default {
@@ -217,24 +241,23 @@ namespace eval ::autoupdate {
 			}
 		}
 
-		destroy .tlsprogress
+		destroy $w
 		::http::cleanup $token
 	}
 
-	proc downloadTLSProgress {url token {total 0} {current 0} } {
-
-		if { $total == 0 } {
-			::autoupdate::errorDownloadingTLS "Couldn't get $url"
-			return
+	proc downloadTLSProgress {w url token {total 0} {current 0} } {
+		if {$total > 0 } {
+			::dkfprogress::SetProgress $w.prbar [expr {$current*100/$total}]
+			$w.progress configure -text "[trans receivedbytes [::amsn::sizeconvert $current] [::amsn::sizeconvert $total]]"
 		}
-		::dkfprogress::SetProgress .tlsprogress.prbar [expr {$current*100/$total}]
-		.tlsprogress.progress configure -text "[trans receivedbytes [::amsn::sizeconvert $current] [::amsn::sizeconvert $total]]"
 
 	}
 
-	proc errorDownloadingTLS { errormsg } {
+	proc errorDownloadingTLS { w errormsg } {
 		::amsn::errorMsg "[trans errortls]: $errormsg"
-		catch { destroy .tlsprogress }
+		if {$w != "" } {
+			catch { destroy $w }
+		}
 	}
 
 	#///////////////////////////////////////////////////////////////////////
@@ -248,7 +271,7 @@ namespace eval ::autoupdate {
 		}
 		#Create the window .downloadwindow
 		toplevel $w
-		wm title $w "Download"
+		wm title $w [trans download]
 		
 		#Create 2 frames
 		frame $w.top
@@ -256,12 +279,12 @@ namespace eval ::autoupdate {
 		
 		#Add bitmap and text at the top
 		label $w.top.bitmap -image [::skin::loadPixmap download]
-		label $w.top.text -text "Downloading new amsn version. Please Wait..." -font bigfont
+		label $w.top.text -text [trans downloadingwait] -font bigfont
 		#Get the download URL
 		set amsn_url [::autoupdate::get_download_link $new_version]
 		
 		#Add button at the bottom
-		button $w.bottom.cancel -text "Cancel"
+		button $w.bottom.cancel -text [trans cancel]
 		
 		#Pack all the stuff for the top
 		pack $w.top.bitmap -pady 5
@@ -292,8 +315,8 @@ namespace eval ::autoupdate {
 		#Create the update window
 		toplevel $w
 		wm title $w "[trans newveravailable $tmp_data]"
-		set changeloglink "http://www.amsn-project.net/wiki/index.php/ChangeLog"
-		set homepagelink "http://www.amsn-project.net/"
+		set changeloglink "http://amsn.sourceforge.net/wiki/tiki-index.php?page=ChangeLog"
+		set homepagelink "$::weburl/"
 		#Create the frames
 		frame $w.top
 		frame $w.top.buttons
@@ -313,7 +336,7 @@ namespace eval ::autoupdate {
 		#button $w.bottom.buttons.cancel -text "Cancel" -command "::autoupdate::dont_ask_before;destroy $w"
 		#label $w.bottom.lastbar -image [::skin::loadPixmap greyline]
 		#Checkbox to verify if the user want to have an alert again or just in one week
-		checkbutton $w.bottom.ignoreoneweek -text "Don't ask update again for one week" -variable "dont_ask_for_one_week" -font sboldf
+		checkbutton $w.bottom.ignoreoneweek -text "[trans dontaskweek]" -variable "dont_ask_for_one_week" -font sboldf
 		
 		#Pack all the stuff for the top
 		pack $w.top.bitmap -side top -padx 3m -pady 3m
@@ -344,9 +367,9 @@ namespace eval ::autoupdate {
 		set new_version [string replace $new_version 1 1 "_"]
 		#set new_version [string replace $new_version 1 1 "-"]
 		append amsn_url "http://aleron.dl.sourceforge.net/sourceforge/amsn/amsn-" $new_version
-		if {![catch {tk windowingsystem} wsystem] && $wsystem == "aqua"} {
+		if { [OnDarwin] } {
 			append amsn_url ".dmg"
-		} elseif { $::tcl_platform(platform)=="windows" } {
+		} elseif { [OnWin] } {
 			append amsn_url ".exe"
 		} else { 
 			append amsn_url ".tar.gz"
@@ -365,8 +388,8 @@ namespace eval ::autoupdate {
 		wm protocol $w WM_DELETE_WINDOW "::http::reset $token"
 		#Check if url is valid
 		if { $total == 0 } {
-			$w.top.text configure -text "Couldn't get $url"
-			$w.bottom.cancel configure -text "Close" -command "destroy $w"
+			$w.top.text configure -text [trans cantget $url]
+			$w.bottom.cancel configure -text [trans close] -command "destroy $w"
 			bind $w <<Escape>> "destroy $w"
 			bind $w <<Destroy>> "destroy $w"
 			wm protocol $w WM_DELETE_WINDOW "destroy $w"
@@ -383,8 +406,8 @@ namespace eval ::autoupdate {
 		#If user cancel the download
 		if { [::http::status $token] == "reset" } {
 			::http::cleanup $token
-			$w.top.text configure -text "Download canceled."
-			$w.bottom.cancel configure -text "Close" -command "destroy $w"
+			$w.top.text configure -text [trans downloadcanceled]
+			$w.bottom.cancel configure -text [trans close] -command "destroy $w"
 			bind $w <<Escape>> "destroy $w"
 			bind $w <<Destroy>> "destroy $w"
 			wm protocol $w WM_DELETE_WINDOW "destroy $w"
@@ -392,8 +415,8 @@ namespace eval ::autoupdate {
 		}
 		#If it's impossible to get the URL
 		if { [::http::status $token] != "ok" || [::http::ncode $token] != 200 } {
-			$w.top.text configure -text "Couldn't get $url"
-			$w.bottom.cancel configure -text "Close" -command "destroy $w"
+			$w.top.text configure -text [trans cantget $url]
+			$w.bottom.cancel configure -text [trans close] -command "destroy $w"
 			bind $w <<Escape>> "destroy $w"
 			bind $w <<Destroy>> "destroy $w"
 			wm protocol $w WM_DELETE_WINDOW "destroy $w"
@@ -408,10 +431,10 @@ namespace eval ::autoupdate {
 		set namelocation [lindex $location 0]
 		set defaultlocation [lindex $location 1]
 		
-		$w.top.text configure -text "Save file on $namelocation?"
+		$w.top.text configure -text [trans savelocation $namelocation]
 		#Create 2 buttons, save and save as
-		button $w.bottom.save -command "::autoupdate::amsn_save $url $token $defaultlocation" -text "Save" -default active
-		button $w.bottom.saveas -command "::autoupdate::amsn_save_as $url $token $defaultlocation" -text "Save in another directory" -default normal
+		button $w.bottom.save -command "::autoupdate::amsn_save $url $token $defaultlocation" -text [trans save] -default active
+		button $w.bottom.saveas -command "::autoupdate::amsn_save_as $url $token $defaultlocation" -text [trans saveotherdirectory] -default normal
 		pack $w.bottom.save
 		pack $w.bottom.saveas -pady 5
 		#If user try to close the window, just save in default directory
@@ -434,10 +457,10 @@ namespace eval ::autoupdate {
 	#(It's ok For Mac OS X, change it on your platform if you feel it's not good)
 	proc get_default_location {} {
 		global env
-		if {![catch {tk windowingsystem} wsystem] && $wsystem == "aqua"} {
+		if { [OnDarwin] } {
 			set namelocation "Desktop"
 			set defaultlocation "[file join $env(HOME) Desktop]"
-		} elseif { $::tcl_platform(platform)=="windows" } {
+		} elseif { [OnWin] } {
 			set namelocation "Received files folder"
 			set defaultlocation "[::config::getKey receiveddir]"
 		} else { 
@@ -453,8 +476,15 @@ namespace eval ::autoupdate {
 	#If yes, save the actual time in seconds in the weekdate
 	proc dont_ask_before {} {
 		if {$::dont_ask_for_one_week} {
-			::config::setKey weekdate "[clock seconds]"
+			#re-check again in a week
+			set wait_period [expr {60*60*24*7}]
+		} else {
+			#just wait one day
+			set wait_period [expr {60*60*24*1}]
 		}
+		set current_time [clock seconds]
+		set next_check [expr {$current_time + $wait_period}]
+		::config::setGlobalKey next_update_check $next_check
 	}
 
 	#///////////////////////////////////////////////////////////////////////
@@ -479,23 +509,23 @@ namespace eval ::autoupdate {
 			set namelocation [lindex $location 0]
 			set defaultlocation [lindex $location 1]
 			#Show the button to choose a new file location or use default location
-			$w.top.text configure -text "File can't be saved at this place."
-			$w.bottom.save configure -command "::autoupdate::amsn_save $url $token $defaultlocation" -text "Save in default location" -default active
-			$w.bottom.saveas configure -command "::autoupdate::amsn_save_as $url $token $defaultlocation" -text "Choose new file location" -default normal
+			$w.top.text configure -text [trans savelocationerror]
+			$w.bottom.save configure -command "::autoupdate::amsn_save $url $token $defaultlocation" -text [trans savedefaultloc] -default active
+			$w.bottom.saveas configure -command "::autoupdate::amsn_save_as $url $token $defaultlocation" -text [trans chooselocation] -default normal
 			
 			bind $w <<Escape>> "::autoupdate::amsn_save $url $token $defaultlocation"
 			bind $w <<Destroy>> "::autoupdate::amsn_save $url $token $defaultlocation"
 			wm protocol $w WM_DELETE_WINDOW "::autoupdate::amsn_save $url $token $defaultlocation"
 		} else {
 			#The saving is a sucess, show a button to open directory of the saved file and close button
-			$w.top.text configure -text "Done\n Saved $fname in $savedir."
-			$w.bottom.save configure -command "launch_filemanager \"$savedir\";destroy $w" -text "Open directory" -default normal
-			$w.bottom.saveas configure -command "destroy $w" -text "Close" -default active
-			#if { $::tcl_platform(platform)=="unix" } {
+			$w.top.text configure -text [trans updatesaved $fname $savedir]
+			$w.bottom.save configure -command "launch_filemanager \"$savedir\";destroy $w" -text [trans opendir] -default normal
+			$w.bottom.saveas configure -command "destroy $w" -text [trans close] -default active
+			#if { [OnLinux] } {
 			#	button $w.bottom.install -text "Install" -command "amsn_install_linux $savedir $fname"
 			#	pack $w.bottom.install
 			#}
-			#if { $::tcl_platform(platform)=="windows" } {
+			#if { [OnWin] } {
 			#	button $w.bottom.install -text "Install" -command "amsn_install_windows $savedir $fname"
 			#	pack $w.bottom.install
 			#}
@@ -503,14 +533,13 @@ namespace eval ::autoupdate {
 			bind $w <<Destroy>> "destroy $w"
 			wm protocol $w WM_DELETE_WINDOW "destroy $w"
 		}
-
 	}
 
 	#///////////////////////////////////////////////////////////////////////
-	package require http
 
 	proc check_web_version { token } {
-		global rcversion weburl
+		global version rcversion weburl
+		if { ![info exists rcversion] } { set rcversion $version }
 
 		set newer 0
 
@@ -531,34 +560,30 @@ namespace eval ::autoupdate {
 			}
 
 			catch {status_log "check_web_ver: Current= $rcversion New=$tmp_data\n"}
-			#Time in second when the user clicked to not have an alert before 3 days
-			set weekdate [::config::getKey weekdate]
-			#Actual time in seconds
-			set actualtime "[clock seconds]"
-			#Number of seconds for 7 days
-			set three_days "[expr {60*60*24*7}]"
-			#If you tant to test just with 60 seconds, add # on the previous line and remove the # on the next one
-			#set three_days "60"
-			#Compare the difference betwen actualtime and the time when he clicked
-			if {$weekdate != ""} {
-				set diff_time "[expr {$actualtime-$weekdate}]"
-			} else {
-				set diff_time "[expr {$three_days + 1 } ]"
-			}
-			status_log "Three days (in seconds) :$three_days\n" blue
-			status_log "Difference time (in seconds): $diff_time\n" blue
-			#If new version and more than 7 days since the last alert (if user choosed that feature)
-			#Open the update window
-			if { $newer == 1 && $diff_time > $three_days} {
+			#Open the update window if newer version
+			if { $newer == 1 } {
 				::autoupdate::update_window $tmp_data
 			} else {
-				status_log "Not yet 3 days or no new version\n" red 
+				status_log "No new version\n" red 
+
+				#no new version, so re-check in a week
+				set current_time [clock seconds]
+				set wait_period [expr {60*60*24*7}]
+				set next_check [expr {$current_time + $wait_period}]
+				::config::setGlobalKey next_update_check $next_check
 			}
-
-
 		} else {
 			catch {status_log "check_web_ver: status=[::http::status $token] ncode=[::http::ncode $token]\n" blue}
-
+			
+			#checking for new version failed, so retry in 1 day
+			set current_time [clock seconds]
+			set wait_period [expr {60*60*24*1}]
+			set next_check [expr {$current_time + $wait_period}]
+			::config::setGlobalKey next_update_check $next_check
+			
+			#abort it, don't even try to update the plugins as it seems that the server has an error
+			::http::cleanup $token
+			return 0
 		}
 		::http::cleanup $token
 
@@ -581,7 +606,6 @@ namespace eval ::autoupdate {
 	proc check_version {} {
 		global weburl
 
-
 		if { [winfo exists .checking] } {
 			raise .checking
 			return
@@ -603,7 +627,7 @@ namespace eval ::autoupdate {
 
 		status_log "Getting ${weburl}/amsn_latest\n" blue
 		if { [catch {
-			set token [::http::geturl ${weburl}/amsn_latest -timeout 10000]
+			set token [::http::geturl ${weburl}/amsn_latest -timeout 5000]
 			if {[::autoupdate::check_web_version $token]==0} {
 				msg_box "[trans nonewver]"
 			}
@@ -619,10 +643,14 @@ namespace eval ::autoupdate {
 	proc check_version_silent {} {
 		global weburl
 
-		catch {
-			::http::geturl ${weburl}/amsn_latest -timeout 10000 -command ::autoupdate::check_web_version
+		#check if check for new version is necessary
+		set next_check [::config::getGlobalKey next_update_check]
+		set current_time [clock seconds]
+		if {$next_check == "" || $next_check < $current_time} {
+			catch {
+				::http::geturl ${weburl}/amsn_latest -timeout 10000 -command ::autoupdate::check_web_version
+			}
 		}
-
 	}
 
 
@@ -647,14 +675,14 @@ namespace eval ::autoupdate {
 
 		toplevel $w
 		wm title $w "[trans update]"
-		wm geometry $w 320x400
+		#wm geometry $w 320x400
 		wm protocol $w WM_DELETE_WINDOW "::autoupdate::UpdateLangPlugin_close"
 		
 		bind $w <<Escape>> "::autoupdate::UpdateLangPlugin_close"
 
 		frame $w.text
 		label $w.text.img -image [::skin::loadPixmap download]
-		label $w.text.txt -text "New updates available for aMSN" -font sboldf
+		label $w.text.txt -text [trans newupdate] -font sboldf
 		pack configure $w.text.img -side left
 		pack configure $w.text.txt -expand true -side right
 		pack $w.text -side top -fill x
@@ -815,6 +843,4 @@ namespace eval ::autoupdate {
 		return [lindex $lang $id]
 
 	}
-
-
 }
