@@ -1,5 +1,13 @@
+
+::Version::setSubversionId {$Id: bugs.tcl 10610 2008-10-23 13:28:12Z vivia $}
+
 proc bgerror { args } {
-    ::bugs::bgerror $args
+	set err $::errorInfo
+	if { [catch {::bugs::bgerror $args} res ] } {
+		puts "Original stack trace :\n$err"
+		puts "\nStack trace of bgerror :\n$::errorInfo"
+		error $res
+	}
 }
 
 namespace eval ::bugs {
@@ -54,7 +62,7 @@ namespace eval ::bugs {
 	}
 
     proc bgerror { args } {
-	global errorInfo errorCode HOME2 tcl_platform tk_patchLevel tcl_patchLevel
+	global errorInfo errorCode HOME2 tcl_platform tk_patchLevel tcl_patchLevel vendor
 	variable dont_give_bug_reports
 	
 	if { [lindex $args 0] == [list] } {
@@ -73,19 +81,18 @@ namespace eval ::bugs {
 	set prot_pos "[expr {[lindex $prot_posend 0]-50}].[lindex $prot_posend 1]"
 	set prot_posend "[lindex $prot_posend 0].[lindex $prot_posend 1]"
 
-	set ::bugs::bug(text) $args
+	set ::bugs::bug(text) [htmlentities $args]
 	set ::bugs::bug(code) $errorCode
-	set ::bugs::bug(info) [privacy $errorInfo]
+	set ::bugs::bug(info) [privacy [htmlentities $errorInfo]]
 	set ::bugs::bug(status) [privacy [htmlentities [.status.info get $pos $posend]]]
 	set ::bugs::bug(protocol) [privacy [htmlentities [.degt.mid.txt get $prot_pos $prot_posend]]]
 	set ::bugs::bug(comment) ""
 	set ::bugs::bug(msnprotocol) [::config::getKey protocol]
+	set ::bugs::bug(loadedplugins) $::plugins::loadedplugins
+	set ::bugs::bug(vendor) $vendor
 
-	if {[file exists cvs_date]==1} {
-	    set fd [open cvs_date]
-	    set date [gets $fd]
-	    close $fd
-	    set date [::bugs::cvstostamp $date]
+	if {$::Version::amsn_revision > 0} {
+	    set date $::Version::date
 	} else {
 	    set date  [clock scan "$::date 00:00:00"]
 	}
@@ -94,14 +101,14 @@ namespace eval ::bugs {
 
 	#error message into status_log
 	status_log "-----------------------------------------\n" error
-	status_log ">>> GOT TCL/TK ERROR : $args\n>>> Stack:\n$::bugs::bug(info)\n>>> Code: $::bugs::bug(code)\n" error
+	status_log ">>> GOT TCL/TK ERROR : $args\n>>> Stack:\n[privacy $errorInfo]\n>>> Code: $::bugs::bug(code)\n" error
 	status_log "-----------------------------------------\n" error
-	catch { status_log ">>> AMSN version: $::version - AMSN date: $::date\n" error }
+	catch { status_log ">>> AMSN version: $::version - AMSN date: $date\n" error }
 	catch { status_log ">>> TCL version : $tcl_patchLevel - TK version : $tk_patchLevel\n" error }
 	catch { status_log ">>> tcl_platform array content : [array get tcl_platform]\n" error }
 	status_log "-----------------------------------------\n\n" error
 
-	::bugs::show_bug_dialog $::bugs::bug(info)
+	::bugs::show_bug_dialog [privacy $errorInfo]
     }
 
     proc save {path} {
@@ -125,12 +132,15 @@ namespace eval ::bugs {
 	puts $fd "\t</error>"
 	puts $fd "\t<system>"
 	puts $fd "\t\t<amsn>$::version</amsn>"
+	puts $fd "\t\t<revision>$::Version::amsn_revision</revision>"
 	puts $fd "\t\t<date>$bug(date)</date>"
 	puts $fd "\t\t<tcl>$tcl_patchLevel</tcl>\n\t\t<tk>$tk_patchLevel</tk>"
 	foreach {key value} [array get tcl_platform] {
 	    puts $fd "\t\t<[string tolower $key]>$value</[string tolower $key]>"
 	}
 	puts $fd "\t\t<msnprotocol>$bug(msnprotocol)</msnprotocol>"
+	puts $fd "\t\t<loadedplugins>$bug(loadedplugins)</loadedplugins>"
+	puts $fd "\t\t<vendor>$bug(vendor)</vendor>"
 	puts $fd "\t</system>"
 	puts $fd "\t<extra>"
 	puts $fd "\t\t<status_log>"
@@ -376,9 +386,9 @@ namespace eval ::bugs {
 }
 
 proc privacy { str } {
-    regsub -all {[A-Za-z0-9._-]{3}@[A-Za-z0-9.-]+} $str {xxx@sadamsnuser.com} str
+    regsub -all {[A-Za-z0-9._-]{3}@[A-Za-z0-9.-]+} $str {xxx@sad-amsn-user.com} str
     #take care of url encoded ones 
-    regsub -all {[A-Za-z0-9._-]{3}%40[A-Za-z0-9.-]+} $str {xxx%40sadamsnuser.com} str
+    regsub -all {[A-Za-z0-9._-]{3}%40[A-Za-z0-9.-]+} $str {xxx%40sad-amsn-user.com} str
     return $str
 }
 
@@ -387,6 +397,7 @@ proc htmlentities {str} {
     regsub -all {\<} $str {\&lt;} str
     regsub -all {\>} $str {\&gt;} str
     regsub -all {\"} $str {\&quot;} str
+    regsub -all {\'} $str {\&apos;} str
     return $str
 }
 
