@@ -1,5 +1,5 @@
 
-::Version::setSubversionId {$Id: config.tcl 11955 2010-03-04 23:21:59Z alexandernst $}
+::Version::setSubversionId {$Id: config.tcl 12326 2011-05-22 21:22:04Z vivia $}
 
 package require des
 
@@ -65,22 +65,7 @@ namespace eval ::config {
 
 		::config::setKey protocoloverride 0
 
-		#Dir for received files
-		if { [OnDarwin] } {
-		        ::config::setKey receiveddir "[file join $::env(HOME) Desktop]"
-		} elseif { [OnMaemo] } {
-			::config::setKey receiveddir "[file join $::env(HOME) MyDocs amsn_received]"
-		} elseif { [OnUnix] } {
-			::config::setKey receiveddir "[file join $::env(HOME) amsn_received]"
-		} elseif { [OnWin] } {
-			if {[info exists env(USERPROFILE)]} {
-				::config::setKey receiveddir "[file join $::env(USERPROFILE) amsn_received]"
-			} else {
-				::config::setKey receiveddir "[file join [pwd] amsn_received]"
-			}
-		} else {
-			::config::setKey receiveddir "[file join [pwd] amsn_received]"
-		}
+		::config::setKey receiveddir [::config::getDefaultDownloadDir]
 
 		#Some Autodetected options
 		if { [OnDarwin] } {
@@ -380,8 +365,14 @@ namespace eval ::config {
 			[list local tooltips bool tooltips] \
 			[list local emailsincontactlist bool emailsinlist] \
 			[list local leavejoinsinchat bool leavejoinsinchat] \
-			[list local animatenotify bool animatenotify] \
-                        [list local blinktray bool blinktray] \
+			[list local animatenotify bool animatenotify]\
+	        ]
+
+                if { ![OnWin] } {
+                        lappend advanced_options [list local blinktray bool blinktray]
+                }
+
+                lappend advanced_options \
 			[list local enablebanner bool showbanner] \
 			[list local truncatenames bool truncatenames1] \
 			[list local truncatenicks bool truncatenames2] \
@@ -437,8 +428,7 @@ namespace eval ::config {
 			[list local globaloverride bool globaloverride ] \
 			[list local escape_close_cw bool escapeclosescw ] \
 			[list local no_oim_confirmation bool nooimconfirmation ] \
-			[list global disableprofiles bool disableprofiles] \
-		]
+			[list global disableprofiles bool disableprofiles]
 
 		set osspecific_keys [list receiveddir soundcommand browser notifyXoffset \
 					notifyYoffset filemanager openfilecommand usesnack \
@@ -605,6 +595,82 @@ namespace eval ::config {
 
 	}
 
+	proc getDefaultDownloadDir { } {
+	
+		#Dir for received files
+		if { [OnDarwin] } {
+		        return "[file join $::env(HOME) Desktop]"
+		} elseif { [OnWin] } {
+			if {[info exists env(USERPROFILE)]} {
+				return "[file join $::env(USERPROFILE) amsn_received]"
+			} else {
+				return "[file join [pwd] amsn_received]"
+			}
+		} elseif { [OnUnix] } {
+			set path ""
+			if {[catch {
+				if {[info exists ::env(XDG_CONFIG_HOME)] } {
+					set config_dir [set ::env(XDG_CONFIG_HOME)]
+				} else {
+					set config_dir [file join [set ::env(HOME)] .config]
+				}
+				status_log "Opening xdg file in [file join $config_dir user-dirs.dir]"
+				set fd [open [file join $config_dir user-dirs.dirs] r]
+				set data [read $fd]
+				close $fd
+				foreach line [split $data "\n"] {
+					status_log "Parsing line $line"
+					set line [string trim $line]
+					if {[string first "XDG_DOWNLOAD_DIR" $line ] == 0} {
+						status_log "found XDG_DOWNLOAD_DIR"
+						set line [string trim [string range $line 16 end]]
+						if {[string first "=" $line] == 0} {
+							status_log "Found ="
+							set path [string trim [string range $line 1 end]]
+							if {[string first "\"" $path] == 0} {
+								status_log "found first quote"
+								set path [string trim [string range $path 1 end]]
+								set second_quote [string first "\"" $path]
+								status_log "second quote at $second_quote"
+								if {$second_quote != -1} {
+									incr second_quote -1
+									set path [string trim [string range $path 0 $second_quote]]
+									status_log "path is $path"
+								}
+								break;
+							}
+						}
+					}
+				}
+				if {$path != ""} {
+					status_log "Found a path : $path"
+					if {[string first "\$HOME/" $path] == 0} {
+						status_log "Path is relative"
+						set relative 1
+						set path [string range  $path 6 end]
+						set path [file join $::env(HOME) $path]
+					}
+					status_log "Final path is : $path"
+				}
+			}]} {
+				set path ""
+				status_log "Error parsing XDG_DOWNLOAD_DIR" red
+			}
+			if {$path == ""} {
+				if { [OnMaemo] } {
+					return "[file join $::env(HOME) MyDocs amsn_received]"
+				}  else { 
+					return "[file join $::env(HOME) amsn_received]"
+				}
+			} else {
+				return $path
+			}
+			
+		} else {
+			return "[file join [pwd] amsn_received]"
+		}
+	}
+
 	proc get {key} {
 		return [set ::config($key)]
 	}
@@ -1305,7 +1371,7 @@ proc LoginList { action age {email ""} {lock ""} } {
 
 		lockexists {
 			set tmp_list [array get LockList]
-			return [lsearch $tmp_list [LoginList getlock "" "$email"]]
+			return [expr {[lsearch $tmp_list [LoginList getlock "" "$email"]] >= 0}]
 		}
 
 		size {
@@ -1446,6 +1512,7 @@ proc SwitchToDefaultProfile { } {
 	::config::setKey log_event_nick 0
 	::config::setKey log_event_psm 0
 	::config::setKey displaypic "amsn.png"
+	::config::setKey receiveddir [::config::getDefaultDownloadDir]
 
 	foreach file [glob -nocomplain -type f [file join $::HOME2 displaypic *]] {
 		catch {file delete $file}
