@@ -1,5 +1,5 @@
 
-::Version::setSubversionId {$Id: msnp2p.tcl 11879 2010-01-09 06:39:00Z kakaroto $}
+::Version::setSubversionId {$Id: msnp2p.tcl 12293 2011-02-14 20:52:44Z vivia $}
 
 ::snit::type P2PMessage {
 
@@ -362,7 +362,14 @@ namespace eval ::MSNP2P {
 	proc SendVoiceClip {chatid file } {
 		set sbn [::MSN::SBFor $chatid]
 		set msnobj [create_msnobj [::config::getKey login] 11 $file]
-		
+   set fd [open $file r]
+   fconfigure $fd -translation {binary binary}
+   set data [read $fd]
+   close $fd
+                set p2pmsnobj [::p2p::MSNObject parse $msnobj]
+                $p2pmsnobj configure -data $data
+                $::obj_stor publish $p2pmsnobj
+
 		set msg "MIME-Version: 1.0\r\nContent-Type: text/x-msnmsgr-datacast\r\n\r\nID: 3\r\nData: $msnobj\r\n\r\n"
 		set msg_len [string length $msg]
 	
@@ -1300,6 +1307,14 @@ namespace eval ::MSNP2P {
 	}
 
 	proc RequestObjectEx { chatid dest msnobject type} {
+
+                #@@@@@@@@@@TODO: callback!!!
+   status_log "Requesting $msnobject"
+   $::obj_stor request [::p2p::MSNObject parse $msnobject] [list ::amsn::WriteNewData]
+
+ }
+
+ proc RequestObjectExOld  { chatid dest msnobject type} {
 		# Let's create a new session
 		set sid [expr {int([expr rand() * 1000000000])%125000000 + 4 } ]
 		# Generate BranchID and CallID
@@ -1748,6 +1763,12 @@ namespace eval ::MSNP2P {
 			set MsgId [lindex [SessionList get $sid] 0]			
 			setObjOption $sid data_blob_id $MsgId
 		}
+
+                if { [$sbn cget -killme] != "" } {
+                        after cancel [$sbn cget -killme]
+                }
+		$sbn configure -killme [after 60000 "::MSN::CloseInactiveSB $sbn"]
+
 	}
 	
 	proc SetSendDataFileEvent { sock sbn sid fd match} { 
