@@ -1,24 +1,23 @@
 #This is the TKCximage implementation inside aMSN
 #By Jerome Gagnon-Voyer gagnonje5000 at mac.com
 
+::Version::setSubversionId {$Id: picture.tcl 11623 2009-09-23 19:39:08Z kakaroto $}
+
 namespace eval ::picture {
 	catch {package require TkCximage}
 	
 	set ::tkcximageloaded 0
 	#Proc to check if tkcximage is loaded
 	proc Loaded {} {
-		global tcl_platform
 		if {$::tkcximageloaded} {
 			return 1
 		} else {
 			catch {package require TkCximage} err
 			#Fix a strange bug where sometimes package require TkCximage doesn't work
-			if {![catch {tk windowingsystem} wsystem] && $wsystem == "aqua"} {
-				catch {load [file join utils macosx TkCximage TkCximage.dylib]} err
-			} elseif { $tcl_platform(platform) == "windows"} {
-				catch {load [file join utils TkCximage TkCximage.dll]} err
+			if {[file exists [file join utils TkCximage [info tclversion] TkCximage[info shared]]]} {
+				catch {load [file join utils TkCximage [info tclversion] TkCximage[info shared]]} err
 			} else {
-				catch {load [file join utils TkCximage TkCximage.so]} err
+				catch {load [file join utils TkCximage TkCximage[info shared]]} err
 			}
 
 			# We try to create an image of format cximage, if TkCximage was loaded, then it will work, if not, it will fail.
@@ -30,6 +29,8 @@ namespace eval ::picture {
 
 		}
 		#puts "Picture.tcl: TkCximage not loaded\n$err"
+                tk_messageBox -default ok -message "There's a problem loading a module of aMSN (TkCxImage) : $err" -icon warning
+
 		return 0
 		
 	}
@@ -145,6 +146,10 @@ namespace eval ::picture {
 			set origh [image height $photo]
 			#status_log "picture.tcl: Image size is $origw $origh\n" red
 			
+			if {$origw == 0 || $origh == 0 } {
+				$photo configure -width $width -height $height
+				return 1
+			}
 			#Actual ratio of the photo
 			set origratio [expr { 1.0*$origw / $origh } ]
 			#New ratio
