@@ -4,6 +4,8 @@
 ##                                                 ##
 #####################################################
 
+::Version::setSubversionId {$Id: plugins.tcl 12084 2010-07-19 15:12:42Z vivia $}
+
 proc plugins_log {plugin msg} {
 #return
 	if {[info procs "::pluginslog::plugins_log"] == "::pluginslog::plugins_log"} {
@@ -192,17 +194,18 @@ namespace eval ::plugins {
 	}
 
 	###############################################################
-	# calledFrom ()
-	#	
-	# Finds out if a proc was called by a plugin.
+        # calledFrom ()
+        #
+        # Finds out if a proc was called by a plugin.
 	#
-	# Arguments
-	# none
-	#
-	# Return
-	# -1 - not called by a plugin
+        # Arguments
+        # none
+        #
+        # Return
+        # -1 - not called by a plugin
 	# $pluginnamespace - the namespace of the plugin calling the proc
-	#
+        #
+
 	proc calledFrom {} {
 	    set namespace [string trimleft "[uplevel 2 namespace current]" "::"]
 	    
@@ -215,17 +218,18 @@ namespace eval ::plugins {
 	}
 
 	###############################################################
-	# namespaceExists (namespace)
-	#
+        # namespaceExists (namespace)
+        #
 	# finds out if a namespace belongs to a plugin
-	#	
-	# Arguments
-	# namespace - namespace to check for (without ::)
-	#	
-	# Return
+        #
+        # Arguments
+        # namespace - namespace to check for (without ::)
+        #
+        # Return
 	# -1 - nope
 	# 1 - yup
-	#
+        #
+
 	proc namespaceExists {namespace} {
 		variable plugins
 		# go through each namespace
@@ -250,7 +254,8 @@ namespace eval ::plugins {
 	#
 	# Return
 	# string - the value of the parameter, empty if not found
-	#	
+	#
+	
 	proc getInfo {plugin param} {
 		variable plugins
 		plugins_log core "Getting $plugin and $param"
@@ -266,10 +271,10 @@ namespace eval ::plugins {
 	}
 
 	###############################################################
-	# getPlugins ()
-	#
+        # getPlugins ()
+        #
 	# Returns a list of existing plugins
-	#
+        #
 	# Arguments
 	# none
 	#
@@ -285,20 +290,21 @@ namespace eval ::plugins {
 	}
 
 	###############################################################
-	# updatePluginsArray ()
-	#
-	# Updates the plugins array which holds info about plugins
+        # updatePluginsArray ()
+        #
+        # Updates the plugins array which holds info about plugins
 	# by searching possible plugin directories
-	#
-	# Arguments
-	# none
-	#	
-	# Return
-	# none
-	#	
+        #
+        # Arguments
+        # none
+        #
+        # Return
+        # none
+        #
+	
 	proc updatePluginsArray { } {
 
-	       	global HOME HOME2
+	       	global HOME HOME2 program_dir
 
 		#clear the current array
 		array set ::plugins::plugins [list]
@@ -306,7 +312,7 @@ namespace eval ::plugins {
 		# make a list of all the possible places to search
 		#TODO: Allow user to choose where to search
 		set search_path [list] 
-		lappend search_path [file join [set ::program_dir] plugins]
+		lappend search_path [file join $program_dir plugins]
 		lappend search_path [file join $HOME plugins]
 		if { $HOME != $HOME2} {
 			lappend search_path [file join $HOME2 plugins]
@@ -320,7 +326,6 @@ namespace eval ::plugins {
 			foreach file [glob -nocomplain -directory $dir */plugininfo.xml] {
 				plugins_log core "Found plugin files in $file\n"
 				::plugins::LoadInfo $file
-				::plugins::LoadInfoAutoupdate $file
 			}
 		}
 
@@ -373,6 +378,7 @@ namespace eval ::plugins {
 	# Return
 	# none
 	#
+
 	proc XMLInfo { cstack cdata saved_data cattr saved_attr args } {
 
 		variable plugins
@@ -390,7 +396,7 @@ namespace eval ::plugins {
 		#<description_fr>the description in french</description_fr>
 		#The defaut description must be written in english
 		#The file should be encoded in utf-8
-		set langcode [::config::getGlobalKey language]
+		set langcode [string tolower [::config::getGlobalKey language]]
 		if { ($langcode != "en") && [info exists sdata(${cstack}:description_${langcode}) ] } {
 			set desc $sdata(${cstack}:description_${langcode})
 		} else {
@@ -403,17 +409,29 @@ namespace eval ::plugins {
 		set plugin_namespace $sdata(${cstack}:plugin_namespace)
 		set init $sdata(${cstack}:init_procedure)
 
+		if { ![info exists sdata(${cstack}:cvs_version)] } {
+			set cvs_version ""
+		} else {
+			set cvs_version $sdata(${cstack}:cvs_version)
+		}
+
 		if { ![info exists sdata(${cstack}:deinit_procedure)] } {
 			set deinit ""
 		} else {
 			set deinit $sdata(${cstack}:deinit_procedure)
 		}
-		
+
+		if {([getInfo $name plugin_version] != "" && [DetectNew $plugin_version [getInfo $name plugin_version]]) ||
+		    ([getInfo $name cvs_version] != "" && [DetectNew $cvs_version [getInfo $name cvs_version]])} {
+			# A previously detected plugin with the same name but a newer version is already loaded in memory.
+			return 0
+		}
 		set plugins(${name}_name) $name
 		set plugins(${name}_author) $author
 		set plugins(${name}_description) $desc
 		set plugins(${name}_amsn_version) $amsn_version
 		set plugins(${name}_plugin_version) $plugin_version
+		set plugins(${name}_cvs_version) $cvs_version
 		#dir is the path to pluginsinfo.xml, so we need to use [file dirname] to get the actual dir path
 		set plugins(${name}_plugin_file) [file join [file dirname $dir] $plugin_file]
 		set plugins(${name}_plugin_dir) [file dirname $dir]
@@ -467,7 +485,7 @@ namespace eval ::plugins {
 		wm geometry $w 500x400
 		# create widgets
 		# listbox with all the plugins
-		listbox $w.plugin_list -background "white" -height 15 -yscrollcommand "$w.ys set" -relief flat -highlightthickness 0
+		listbox $w.plugin_list -height 15 -yscrollcommand "$w.ys set" -highlightthickness 0
 		scrollbar $w.ys -command "$w.plugin_list yview"
 
 		#Scrollableframe that will contain pieces of information about a plugin
@@ -487,11 +505,22 @@ namespace eval ::plugins {
 		# TODO make the -wraplength fit the label's width
 		label $mF.desc -wraplength 280 -justify left -anchor w
 		# holds the 'command center' buttons
-		label $w.getmore -text "[trans getmoreplugins]" -fg #0000FF
+		label $w.getmore -text "[trans getmoreplugins]" -cursor hand2 -font splainf \
+			-background [::skin::getKey extrastdwindowcolor] -foreground [::skin::getKey extralinkcolor]
+ 
+		bind $w.getmore <Enter> "$w.getmore configure -font sunderf -cursor hand2 \
+			-background [::skin::getKey extralinkbgcoloractive] -foreground [::skin::getKey extralinkcoloractive]"
+		bind $w.getmore <Leave> "$w.getmore configure -font splainf -cursor left_ptr \
+			-background [::skin::getKey extrastdwindowcolor] -foreground [::skin::getKey extralinkcolor]"
 
-		button $w.load -text "[trans load]" -command "::plugins::GUI_Load" -state disabled
-		button $w.config -text "[trans configure]" -command "::plugins::GUI_Config" ;#-state disabled
-		button $w.close -text [trans close] -command "::plugins::GUI_Close"
+		set lang [::config::getGlobalKey language]
+		bind $w.getmore <ButtonRelease> "launch_browser $::weburl/plugins.php?lang=$lang"
+
+
+		frame $w.bf
+		button $w.bf.load -text "[trans load]" -command "::plugins::GUI_Load" -state disabled
+		button $w.bf.config -text "[trans configure]" -command "::plugins::GUI_Config" -state disabled
+		button $w.bf.close -text [trans close] -command "::plugins::GUI_Close"
  
 		#loop through all the plugins and add them to the list
 		foreach {plugin} [lsort -dictionary [array names ::plugins::plugins *_name]] {
@@ -499,12 +528,11 @@ namespace eval ::plugins {
 		    # add the plugin name to the list at counterid position
 		    $w.plugin_list insert end $name
 		    # if the plugin is loaded, color it one color. otherwise use other colors
-		    #TODO: Why not use skins?
 		    if {[lsearch "$loadedplugins" $plugins(${name}_name)] != -1} {
-			$w.plugin_list itemconfigure end -background #DDF3FE
-		    } else {
-			$w.plugin_list itemconfigure end -background #FFFFFF
-		    }
+				$w.plugin_list itemconfigure end \
+					-bg [::skin::getKey extralistboxselectedbg] -fg [::skin::getKey extralistboxselected] \
+					-selectforeground [::skin::getKey extralistboxselected]
+			}
 		}
 		if {[$w.plugin_list size] > "15"} {
 			$w.plugin_list configure -height [$w.plugin_list size]
@@ -523,15 +551,10 @@ namespace eval ::plugins {
 		pack $mF.author -padx 5 -anchor w
 		pack $mF.desc_title -padx 5 -anchor w
 		pack $mF.desc -anchor nw -expand true -fill x -padx 5
+		pack $w.getmore -side bottom -anchor e -padx 5
+		pack $w.bf.close $w.bf.config $w.bf.load -side right -anchor e -padx 5 -pady 5
+		pack $w.bf -side bottom -anchor e -padx 5 -pady 5
 		pack $w.sw -anchor w -side top -expand true -fill both
-		pack $w.getmore -side top -anchor e -padx 5
-		bind $w.getmore <Enter> "$w.getmore configure -font sunderf"
-		bind $w.getmore <Leave> "$w.getmore configure -font splainf"
-		set lang [::config::getGlobalKey language]
-		bind $w.getmore <ButtonRelease> "launch_browser $::weburl/plugins.php?lang=$lang"
-
-		pack $w.close $w.config $w.load -padx 5 -pady 5 -side right -anchor se
-
 
 		moveinscreen $w 30
 		return
@@ -580,25 +603,30 @@ namespace eval ::plugins {
 		
 		# update the buttons
 
-		$w.config configure -state normal
+		$w.bf.config configure -state normal
 
 		if {[lsearch "$loadedplugins" $selection] != -1 } {
 			# if the plugin is loaded, enable the Unload button and update the colors
-			$w.load configure -state normal -text [trans unload] -command "::plugins::GUI_Unload"
-			$w.plugin_list itemconfigure [$w.plugin_list curselection] -background #DDF3FE
+			$w.bf.load configure -state normal -text [trans unload] -command "::plugins::GUI_Unload"
+			$w.plugin_list itemconfigure [$w.plugin_list curselection] \
+				-bg [::skin::getKey extralistboxselectedbg] -fg [::skin::getKey extralistboxselected] \
+				-selectforeground [::skin::getKey extralistboxselected]
 
 			# if the plugin has a configlist, then enable configuration.
 			# Otherwise disable it
 			if {[info exists ::[getInfo $selection plugin_namespace]::configlist] == 1} {
-				$w.config configure -state normal
+				$w.bf.config configure -state normal
 			} else {
-				$w.config configure -state disabled
+				$w.bf.config configure -state disabled
 			}
 		} else { # plugin is not loaded
 			# enable the load button and disable config button and update color
-			$w.load configure -state normal -text "[trans load]" -command "::plugins::GUI_Load"
-			$w.plugin_list itemconfigure [$w.plugin_list curselection] -background #FFFFFF
-			$w.config configure -state disabled
+			$w.bf.load configure -state normal -text "[trans load]" -command "::plugins::GUI_Load"
+			$w.plugin_list itemconfigure [$w.plugin_list curselection] \
+				-bg [::skin::getKey extrastdbgcolor] -fg [::skin::getKey extrastdtxtcolor] \
+				-selectforeground [::skin::getKey extraselectedtxtcolor]
+
+			$w.bf.config configure -state disabled
 		}
 	}
 
@@ -657,6 +685,12 @@ namespace eval ::plugins {
 		if {$selection==""} {
 		    return;
 		}
+
+		# close configure dialog
+		if {[winfo exists ${w}.winconf_$selection]} {
+			destroy ${w}.winconf_$selection
+		}
+
 		
 		# do the actual unloading
 		UnLoadPlugin $selection
@@ -690,7 +724,7 @@ namespace eval ::plugins {
 	    }
 
 	    #name of the configuration window
-	    set confw ${w}.winconf_${selection}
+	    set confw ${w}.winconf_$selection
 
 	    #If the window is already here, just raise it to the front
 	    if { [winfo exists $confw] } {
@@ -698,7 +732,7 @@ namespace eval ::plugins {
 		return
 	    } else {
 		#create the configuration window
-		set winconf [toplevel $confw]	
+		set winconf [toplevel $confw]
 	    }
 
 	    # list of callbacks for pressing save of frame types
@@ -753,7 +787,7 @@ namespace eval ::plugins {
 				# This configuration item is a text input (Text string variable)
 				set frame [frame $confwin.f$i]
 				entry $frame.${i}e -textvariable \
-				    ::${namespace}::config([lindex $confitem 2]) -bg white
+				    ::${namespace}::config([lindex $confitem 2])
 				label $frame.${i}l -text [lindex $confitem 1]
 				pack $frame.${i}l -anchor w -side left -padx 20
 				pack $frame.${i}e -anchor w -side left -fill x
@@ -772,7 +806,7 @@ namespace eval ::plugins {
 			    lst {
 				# This configuration item is a listbox that stores the selected item.
 				set height [llength [lindex $confitem 1]]
-				listbox $confwin.$i -height $height -width 0 -bg white
+				listbox $confwin.$i -height $height -width 0
 				foreach item [lindex $confitem 1] {
 				    $confwin.$i insert end $item
 				}
@@ -805,7 +839,7 @@ namespace eval ::plugins {
 		}
 		
 		# set the name of the winconf
-		wm title $confw "[trans configure] $selection"
+		wm title $confw [trans configureplugin $selection]
 		
 		# Grid the frame
 		pack $confwin -fill x
@@ -920,11 +954,16 @@ namespace eval ::plugins {
 	# Return
 	# none
 	#
-	proc UnLoadPlugins { {save 1} } {
+	proc UnLoadPlugins { {saveBefore 1} } {
 		variable loadedplugins
+
+		if {$saveBefore} {
+			::plugins::save_config
+		}
+
 		foreach {plugin} "$loadedplugins" {
 			# the "0" is to tell UnLoadPlugin not to save the config
-			::plugins::UnLoadPlugin $plugin $save
+			::plugins::UnLoadPlugin $plugin 0
 		}
 	}
 
@@ -988,22 +1027,12 @@ namespace eval ::plugins {
 	proc LoadPlugins {} {
 
 	    variable loadedplugins
-		# the "0" is to tell UnLoadPlugins not to save the config
-		# the config should not be saved now, but when disconnecting or closing aMSN
+		# the "0" is to tell UnLoadPlugins not to save the config before unloading plugins
 	    ::plugins::UnLoadPlugins 0
 	    
 	    #resets the loadedplugins list to plugins that were loaded before
 	    load_config
 
-	    #"HACK" to load 'core' plugins for the 0.95 release
-	    #HERE WE HAVE A LIST OF PLUGINS THAT ARE SHIPPED WITH AMSN AND SHOULD BE LOADED IF THE USER SEES 'M FOR THE FIRST TIME
-	    #logic: if this is the first time the user logged in, then 
-	    #the configuration list is empty which means load_config
-	    #makes loadedplugins empty
-	    if {[llength $loadedplugins] == 0} {
-		    set loadedplugins [list "Nudge" "Cam Shooter" "remind"]
-	    }
-
 	    #update the list of installed plugins because this proc is usually 
 	    #called when a new user logs in, so he migh have diff plugins in
 	    # his ~/.amsn/{user}/plugins
@@ -1036,8 +1065,8 @@ namespace eval ::plugins {
 	# plugin - The plugin to load (name)
 	# required_version - Required aMSN version for the plugin to load
 	# file - The plugin's TCL main file
-	# namespace - The plugin's main namespace
-	# init_proc - The plugin's init procedure
+        # namespace - The plugin's main namespace
+        # init_proc - The plugin's init procedure
 	#
 	# Return
 	# 1 - success
@@ -1165,13 +1194,20 @@ namespace eval ::plugins {
 	# none
 	#
 	proc save_config { } {
-		global tcl_platform HOME HOME2 version 
+		global HOME HOME2
 		variable loadedplugins
 
-		plugins_log core "save_config: saving plugin config for user [::config::getKey login] in $HOME]\n"
+		plugins_log core "save_config: saving plugin config for user [::config::getKey login] in $HOME\n"
 	
+		# check if this is the default profile
+		if { $HOME == $HOME2 } { 
+			plugins_log core "save_config: Default profile, no plugin configuration saved!\n"
+			catch {file delete -force [file join ${HOME} plugins.xml]}
+			return 0
+		}
+
 		if { [catch {
-			if {$tcl_platform(platform) == "unix"} {
+			if { [OnUnix] } {
 				set file_id [open "[file join ${HOME} plugins.xml]" w 00600]
 			} else {
 				set file_id [open "[file join ${HOME} plugins.xml]" w]
@@ -1180,12 +1216,16 @@ namespace eval ::plugins {
 			return 0
 		}
 
+
 		plugins_log core "save_config: saving plugin config_file. Opening of file returned : $res\n"
 
 		puts $file_id  "<?xml version=\"1.0\"?>\n\n<config>"
 
 		#save the loaded plugins
 		foreach {plugin} $loadedplugins {
+			if { [array names ::plugins::plugins ${plugin}_name] eq "" } {
+				continue
+			}
 			set namespace [getInfo $plugin plugin_namespace]
 			puts $file_id "\t<plugin>"
 			puts $file_id "\t\t<name>${plugin}</name>"
@@ -1209,6 +1249,9 @@ namespace eval ::plugins {
 		
 		#save the other plugins
 		foreach {plugin} [array names ::plugins::config] {
+			if { [array names ::plugins::plugins ${plugin}_name] eq "" } {
+				continue
+			}
 			puts $file_id "\t<plugin>"
 			puts $file_id "\t\t<name>${plugin}</name>"
 			puts $file_id "\t\t<loaded>false</loaded>"
@@ -1242,7 +1285,7 @@ namespace eval ::plugins {
 	# none
 	#
 	proc load_config {} {
-		global HOME password protocol tcl_platform
+		global HOME password
 		variable loadedplugins
 		foreach {plugin} $loadedplugins {
 			::plugins::UnLoadPlugin $plugin
@@ -1264,6 +1307,14 @@ namespace eval ::plugins {
 				file copy [file join ${HOME} "plugins.xml"] [file join ${HOME} "plugins.xml.old"]
 			}
 		} else {
+
+			#"HACK" to load 'core' plugins for the 0.95 release
+			#HERE WE HAVE A LIST OF PLUGINS THAT ARE SHIPPED WITH AMSN AND SHOULD BE LOADED IF THE USER SEES 'M FOR THE FIRST TIME
+			#logic: if this is the first time the user logged in, then 
+			#the configuration list is empty which means load_config
+			#makes loadedplugins empty
+			set loadedplugins [list "Nudge" "Winks" "Cam Shooter" "remind" "Notes" "ColoredNicks" "Inkdraw"]
+
 			status_log "Plugins System: load_config: No plugins.xml]\n" red
 		}
 	}
@@ -1487,7 +1538,7 @@ namespace eval ::plugins {
 	#
 	proc LoadInfoAutoupdate_Online { plugin } {
 
-		global HOME HOME2
+		global HOME2
 		variable plugins
 		variable cur_plugin $plugin
 
@@ -1496,8 +1547,6 @@ namespace eval ::plugins {
 		
 		set URL [getInfo $plugin URL_plugininfo]
 		
-		set program_dir [set ::program_dir]
-
 		if { [catch {
 			
 			# If no URL is given, look at the CVS URL
@@ -1704,13 +1753,10 @@ namespace eval ::plugins {
 	#
 	proc DownloadMain { plugin } {
 	
-		global HOME HOME2
 		variable plugins
 
 		set version [getInfo $plugin cvs_version_online]
 
-		set program_dir [set ::program_dir]
-		
 		set w ".updatelangplugin"
 		
 		if { [winfo exists $w] } {
@@ -1728,20 +1774,20 @@ namespace eval ::plugins {
 			set URL "[subst $URL]"
 			set token [::http::geturl "$URL" -timeout 120000 -binary 1]
 		} else {
-			return 0
+			return -1
 		}
 
 		set status [::http::status $token]
 		if { $status != "ok" } {
 			::http::cleanup $token
-			return 0
+			return -2
 		}
 
 		set content [::http::data $token]
 		
 		if { [string first "<html>" "$content"] != -1 } {
 			::http::cleanup $token
-			return 0
+			return -3
 		}
 
 		set filename [file join [getInfo $plugin plugin_dir] [getInfo $plugin plugin_file]]
@@ -1769,11 +1815,8 @@ namespace eval ::plugins {
 	#
 	proc DownloadLangs { plugin langs } {
 
-		global HOME HOME2
 		variable plugins
 
-		set program_dir [set ::program_dir]
-		
 		set w ".updatelangplugin"
 		
 		foreach lang $langs {
@@ -1813,6 +1856,12 @@ namespace eval ::plugins {
 			}
 
 			set filename [file join [getInfo $plugin plugin_dir] "lang" lang$langcode]
+			set dir [file join [getInfo $plugin plugin_dir] "lang"]
+
+			if { ![file isdirectory $dir] } {
+				file mkdir $dir
+				status_log "Auto-update ($plugin) : create dir $dir\n" red
+			}
 
 			set fid [open $filename w]
 			fconfigure $fid -encoding binary
@@ -1841,11 +1890,8 @@ namespace eval ::plugins {
 	#
 	proc DownloadFiles { plugin files } {
 
-		global HOME HOME2
 		variable plugins
 		
-		set program_dir [set ::program_dir]
-		
 		set w ".updatelangplugin"
 		
 		foreach file_version $files {
@@ -1890,10 +1936,10 @@ namespace eval ::plugins {
 			if { ![file isdirectory $dir] } {
 				file mkdir $dir
 				status_log "Auto-update ($plugin) : create dir $dir\n" red
-			}	
+			}
 
 			set fid [open $filename w]
-			fconfigure $fid -encoding binary
+			fconfigure $fid -encoding binary -translation {binary binary}
 			puts -nonewline $fid "$content"
 			close $fid
 			::http::cleanup $token
@@ -1982,6 +2028,7 @@ namespace eval ::plugins {
 	# Return
 	# 1 if a plugin has been updated, 0 else
 	#
+
 	proc UpdatedPlugins { } {
 
 		variable plugins
@@ -2044,7 +2091,7 @@ namespace eval ::plugins {
 				set version_online [lindex $lang_online 1]
 
 				if { [::lang::LangExists $langcode_online] } {
-
+					
 					foreach lang [getInfo $plugin lang] {
 						set langcode [lindex $lang 0]
 						set version [lindex $lang 1]
@@ -2053,9 +2100,13 @@ namespace eval ::plugins {
 						}
 					}
 
-					if { [::plugins::DetectNew $version $version_online] } {
+					# If $version doesn't exist or if the langcode is different, it means it's a new language
+					# that wasn't in the currently installed version. So it needs to be updated.
+					if { ![info exists version] || 
+					     $langcode != $langcode_online ||
+					     [::plugins::DetectNew $version $version_online] } {
 					
-						set file [file join $path "lang" lang$langcode]
+						set file [file join $path "lang" lang$langcode_online]
 						
 						if { [file exists $file] && ![file writable $file] } {
 							status_log "File $file is protected"
@@ -2088,7 +2139,11 @@ namespace eval ::plugins {
 					}
 				}
 
-				if { [::plugins::DetectNew $version $version_online] } {
+				# If $version doesn't exist or if the pathfile is different, it means it's a new file
+				# that wasn't in the currently installed version. So it needs to be updated.
+				if { ![info exists version] || 
+				     $pathfile != $pathfile_online ||
+				     [::plugins::DetectNew $version $version_online] } {
 					set file [file join $path $pathfile_online]
 					if { [file exists $file] && ![file writable $file] } {
 						status_log "File $file is protected"
