@@ -6,7 +6,7 @@
 ##      section of settings.xml, so the system remains coherent       ##
 ########################################################################
 
-::Version::setSubversionId {$Id: skins.tcl 11592 2009-09-03 04:01:44Z kakaroto $}
+::Version::setSubversionId {$Id: skins.tcl 12325 2011-05-17 20:33:15Z vivia $}
 
 namespace eval ::skin {
 
@@ -642,6 +642,10 @@ namespace eval ::skin {
 				
 		#If we didn't find anything...use null file
 		set path [file join [set ::program_dir] skins $defaultskin $type null]
+		if { ![file exists $path] } {
+			catch {set fd [open $path w]}
+			catch {close $fd}
+		}
 		return $path
 	}
 
