@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 #  tree.tcl
 #  This file is part of Unifix BWidget Toolkit
-#  $Id: tree.tcl 3417 2004-12-03 00:31:24Z tjikkun $
+#  $Id: tree.tcl,v 1.60 2009/07/24 16:01:55 oehhar Exp $
 # ----------------------------------------------------------------------------
 #  Index of commands:
 #     - Tree::create
@@ -10,6 +10,7 @@
 #     - Tree::insert
 #     - Tree::itemconfigure
 #     - Tree::itemcget
+#     - Tree::bindArea
 #     - Tree::bindText
 #     - Tree::bindImage
 #     - Tree::delete
@@ -60,7 +61,7 @@ namespace eval Tree {
             {-data       String     ""      0}
             {-open       Boolean    0       0}
 	    {-selectable Boolean    1       0}
-            {-drawcross  Enum       auto    0 {auto allways never}}
+            {-drawcross  Enum       auto    0 {auto always never allways}}
 	    {-padx       Int        -1      0 "%d >= -1"}
 	    {-deltax     Int        -1      0 "%d >= -1"}
 	    {-anchor     String     "w"     0 ""}
@@ -221,7 +222,6 @@ proc Tree::create { path args } {
     Tree::bindText  $path <Control-Button-1> [list $selectcmd $path toggle]
     Tree::bindImage $path <Control-Button-1> [list $selectcmd $path toggle]
 
-
     # Add sentinal bindings for double-clicking on items, to handle the 
     # gnarly Tk bug wherein:
     # ButtonClick
@@ -327,6 +327,7 @@ proc Tree::insert { path index parent node args } {
     if { [info exists data($node)] } {
         return -code error "node \"$node\" already exists"
     }
+    set parent [_node_name $path $parent]
     if { ![info exists data($parent)] } {
         return -code error "node \"$parent\" does not exist"
     }
@@ -428,13 +429,19 @@ proc Tree::itemcget { path node option } {
     return [Widget::cget $path.$node $option]
 }
 
+# ----------------------------------------------------------------------------
+# Command Tree::bindArea
+# ----------------------------------------------------------------------------
+proc Tree::bindArea { path event script } {
+    bind $path.c $event $script
+}
 
 # ----------------------------------------------------------------------------
 #  Command Tree::bindText
 # ----------------------------------------------------------------------------
 proc Tree::bindText { path event script } {
     if {[string length $script]} {
-	append script " \[Tree::_get_node_name [list $path] current 2\]"
+	append script " \[Tree::_get_node_name [list $path] current 2 1\]"
     }
     $path.c bind "node" $event $script
     if {[Widget::getoption $path -selectfill]} {
@@ -450,7 +457,7 @@ proc Tree::bindText { path event script } {
 # ----------------------------------------------------------------------------
 proc Tree::bindImage { path event script } {
     if {[string length $script]} {
-	append script " \[Tree::_get_node_name [list $path] current 2\]"
+	append script " \[Tree::_get_node_name [list $path] current 2 1\]"
     }
     $path.c bind "img" $event $script
     if {[Widget::getoption $path -selectfill]} {
@@ -468,6 +475,7 @@ proc Tree::delete { path args } {
     variable $path
     upvar 0  $path data
 
+    set sel 0
     foreach lnodes $args {
 	foreach node $lnodes {
             set node [_node_name $path $node]
@@ -475,10 +483,14 @@ proc Tree::delete { path args } {
 		set parent [lindex $data($node) 0]
 		set idx	   [lsearch -exact $data($parent) $node]
 		set data($parent) [lreplace $data($parent) $idx $idx]
-		_subdelete $path [list $node]
+		incr sel [_subdelete $path [list $node]]
 	    }
 	}
     }
+    if {$sel} {
+	# if selection changed, call the selectcommand
+	__call_selectcmd $path
+    }
 
     _redraw_idle $path 3
 }
@@ -693,7 +705,11 @@ proc Tree::selection { path cmd args } {
 		return -code error \
 			"wrong#args: Expected $path selection get"
 	    }
-            return $data(selnodes)
+            set nodes [list]
+	    foreach node $data(selnodes) {
+		lappend nodes [_node_name_rev $path $node]
+	    }
+	    return $nodes
         }
         includes {
 	    if {[llength $args] != 1} {
@@ -821,16 +837,16 @@ proc Tree::find {path findInfo {confine ""}} {
     }
 
     if {$found} {
-        if {[string equal $confine "confine"]} {
+        if {![string equal $confine ""]} {
             # test if x stand inside node bbox
 	    set padx [_get_node_padx $path $node]
             set xi [expr {[lindex [$path.c coords n:$node] 0] - $padx}]
             set xs [lindex [$path.c bbox n:$node] 2]
             if {$x >= $xi && $x <= $xs} {
-                return $node
+                return [_node_name_rev $path $node]
             }
         } else {
-            return $node
+            return [_node_name_rev $path $node]
         }
     }
     return ""
@@ -839,7 +855,7 @@ proc Tree::find {path findInfo {confine ""}} {
 
 # ----------------------------------------------------------------------------
 #  Command Tree::line
-#     Returns the line where is drawn a node.
+#     Returns the line where a node was drawn.
 # ----------------------------------------------------------------------------
 proc Tree::line {path node} {
     set node [_node_name $path $node]
@@ -1048,7 +1064,7 @@ proc Tree::edit { path node text {verifycmd ""} {clickres 0} {select 1}} {
         set ok 0
         while { !$ok } {
             tkwait variable Tree::_edit(wait)
-            if { !$_edit(wait) || $verifycmd == "" ||
+            if { !$_edit(wait) || [llength $verifycmd]==0 ||
                  [uplevel \#0 $verifycmd [list $_edit(text)]] } {
                 set ok 1
             }
@@ -1075,7 +1091,7 @@ proc Tree::edit { path node text {verifycmd ""} {clickres 0} {select 1}} {
 #  Command Tree::xview
 # ----------------------------------------------------------------------------
 proc Tree::xview { path args } {
-    return [eval [list $path.c xview] $args]
+    return [eval [linsert $args 0 $path.c xview]]
 }
 
 
@@ -1083,7 +1099,7 @@ proc Tree::xview { path args } {
 #  Command Tree::yview
 # ----------------------------------------------------------------------------
 proc Tree::yview { path args } {
-    return [eval [list $path.c yview] $args]
+    return [eval [linsert $args 0 $path.c yview]]
 }
 
 
@@ -1141,7 +1157,7 @@ proc Tree::_recexpand { path node expand recursive cmd } {
 
     if { [Widget::getoption $path.$node -open] != $expand } {
         Widget::setoption $path.$node -open $expand
-        if { $cmd != "" } {
+        if {[llength $cmd]} {
             uplevel \#0 $cmd [list $node]
         }
     }
@@ -1162,6 +1178,7 @@ proc Tree::_subdelete { path lnodes } {
     upvar 0  $path data
 
     set sel $data(selnodes)
+    set selchanged 0
 
     while { [llength $lnodes] } {
         set lsubnodes [list]
@@ -1173,6 +1190,7 @@ proc Tree::_subdelete { path lnodes } {
 	    set idx [lsearch -exact $sel $node]
 	    if { $idx >= 0 } {
 		set sel [lreplace $sel $idx $idx]
+		incr selchanged
 	    }
             if { [set win [Widget::getoption $path.$node -window]] != "" } {
                 destroy $win
@@ -1183,6 +1201,8 @@ proc Tree::_subdelete { path lnodes } {
     }
 
     set data(selnodes) $sel
+    # return number of sel items changes
+    return $selchanged
 }
 
 
@@ -1232,12 +1252,12 @@ proc Tree::_cross_event { path } {
     set node [Tree::_get_node_name $path current 1]
     if { [Widget::getoption $path.$node -open] } {
         Tree::itemconfigure $path $node -open 0
-        if { [set cmd [Widget::getoption $path -closecmd]] != "" } {
+        if {[llength [set cmd [Widget::getoption $path -closecmd]]]} {
             uplevel \#0 $cmd [list $node]
         }
     } else {
         Tree::itemconfigure $path $node -open 1
-        if { [set cmd [Widget::getoption $path -opencmd]] != "" } {
+        if {[llength [set cmd [Widget::getoption $path -opencmd]]]} {
             uplevel \#0 $cmd [list $node]
         }
     }
@@ -1285,7 +1305,6 @@ proc Tree::_draw_cross { path node open x y } {
 #  Command Tree::_draw_node
 # ----------------------------------------------------------------------------
 proc Tree::_draw_node { path node x0 y0 deltax deltay padx showlines } {
-    global   env
     variable $path
     upvar 0  $path data
 
@@ -1312,7 +1331,8 @@ proc Tree::_draw_node { path node x0 y0 deltax deltay padx showlines } {
                     [expr {$x0+$deltax}] $y0 $deltax $deltay $padx $showlines]
     }
 
-    if {![string equal $dc "never"] && ($len || [string equal $dc "allways"])} {
+    if {![string equal $dc "never"]
+	&& ($len || [string equal $dc "always"] || [string equal $dc "allways"])} {
         _draw_cross $path $node $exp $x0 $y0
     }
 
@@ -1349,7 +1369,10 @@ proc Tree::_draw_subnodes { path nodes x0 y0 deltax deltay padx showlines } {
         set yp $y1
         set y1 [_draw_node $path $node $x0 [expr {$y1+$deltay}] $deltax $deltay $padx $showlines]
     }
-    if { $showlines && [llength $nodes] } {
+    # Only draw a line to the invisible root node above the tree widget when
+    # there are multiple top nodes.
+    set len [llength $nodes]
+    if { $showlines && $len && !($y0 < 0 && $len < 2) } {
         set id [$path.c create line $x0 $y0 $x0 [expr {$yp+$deltay}] \
                     -fill    [Widget::getoption $path -linesfill]   \
                     -stipple [Widget::getoption $path -linestipple] \
@@ -1365,73 +1388,70 @@ proc Tree::_draw_subnodes { path nodes x0 y0 deltax deltay padx showlines } {
 #  Command Tree::_update_nodes
 # ----------------------------------------------------------------------------
 proc Tree::_update_nodes { path } {
-    global   env
     variable $path
     upvar 0  $path data
 
-    set deltax [Widget::getoption $path -deltax]
-    set padx   [Widget::getoption $path -padx]
     foreach {node flag} $data(upd,nodes) {
-        set idn [$path.c find withtag "n:$node"]
-        if { $idn == "" } {
-            continue
-        }
+	set idn [$path.c find withtag "n:$node"]
+	if { $idn == "" } {
+	    continue
+	}
 	set padx   [_get_node_padx $path $node]
 	set deltax [_get_node_deltax $path $node]
-        set c  [$path.c coords $idn]
-        set x0 [expr {[lindex $c 0]-$padx}]
-        set y0 [lindex $c 1]
-        if { $flag & 48 } {
-            # -window or -image modified
-            set win  [Widget::getoption $path.$node -window]
-            set img  [Widget::getoption $path.$node -image]
-            set idi  [$path.c find withtag i:$node]
-            set type [lindex [$path.c gettags $idi] 1]
-            if { [string length $win] } {
-                if { [string equal $type "win"] } {
-                    $path.c itemconfigure $idi -window $win
-                } else {
-                    $path.c delete $idi
-                    $path.c create window $x0 $y0 -window $win -anchor w \
-			    -tags [Tree::_get_node_tags $path $node \
-			    	[list win i:$node]]
-                }
-            } elseif { [string length $img] } {
-                if { [string equal $type "img"] } {
-                    $path.c itemconfigure $idi -image $img
-                } else {
-                    $path.c delete $idi
-                    $path.c create image $x0 $y0 -image $img -anchor w \
-			    -tags [Tree::_get_node_tags $path $node \
-			    	[list img i:$node]]
-                }
-            } else {
-                $path.c delete $idi
-            }
-        }
+	set c  [$path.c coords $idn]
+	set x1 [expr {[lindex $c 0]-$padx}]
+	set x0 [expr {$x1-$deltax-5}]
+	set y0 [lindex $c 1]
+	if { $flag & 48 } {
+	    # -window or -image modified
+	    set win  [Widget::getoption $path.$node -window]
+	    set img  [Widget::getoption $path.$node -image]
+	    set anc  [Widget::cget $path.$node -anchor]
+	    set idi  [$path.c find withtag i:$node]
+	    set type [lindex [$path.c gettags $idi] 1]
+	    if { [string length $win] } {
+		if { [string equal $type "win"] } {
+		    $path.c itemconfigure $idi -window $win
+		} else {
+		    $path.c delete $idi
+		    $path.c create window $x1 $y0 -window $win -anchor $anc \
+			-tags [_get_node_tags $path $node [list win i:$node]]
+		}
+	    } elseif { [string length $img] } {
+		if { [string equal $type "img"] } {
+		    $path.c itemconfigure $idi -image $img
+		} else {
+		    $path.c delete $idi
+		    $path.c create image $x1 $y0 -image $img -anchor $anc \
+			-tags [_get_node_tags $path $node [list img i:$node]]
+		}
+	    } else {
+		$path.c delete $idi
+	    }
+	}
 
-        if { $flag & 8 } {
-            # -drawcross modified
-            set len [expr {[llength $data($node)] > 1}]
-            set dc  [Widget::getoption $path.$node -drawcross]
-            set exp [Widget::getoption $path.$node -open]
+	if { $flag & 8 } {
+	    # -drawcross modified
+	    set len [expr {[llength $data($node)] > 1}]
+	    set dc  [Widget::getoption $path.$node -drawcross]
+	    set exp [Widget::getoption $path.$node -open]
 
-            if {![string equal $dc "never"]
-                && ($len || [string equal $dc "allways"])} {
-                _draw_cross $path $node $exp $x0 $y0
-            } else {
+	    if {![string equal $dc "never"]
+		&& ($len || [string equal $dc "always"] || [string equal $dc "allways"])} {
+		_draw_cross $path $node $exp $x0 $y0
+	    } else {
 		set idc [$path.c find withtag c:$node]
-                $path.c delete $idc
-            }
-        }
+		$path.c delete $idc
+	    }
+	}
 
-        if { $flag & 7 } {
-            # -font, -text or -fill modified
-            $path.c itemconfigure $idn \
-                -text [Widget::getoption $path.$node -text] \
-                -fill [Widget::getoption $path.$node -fill] \
-                -font [Widget::getoption $path.$node -font]
-        }
+	if { $flag & 7 } {
+	    # -font, -text or -fill modified
+	    $path.c itemconfigure $idn \
+		-text [Widget::getoption $path.$node -text] \
+		-fill [Widget::getoption $path.$node -fill] \
+		-font [Widget::getoption $path.$node -font]
+	}
     }
 }
 
@@ -1511,7 +1531,10 @@ proc Tree::_redraw_selection { path } {
             }
             set id [$path.c create rectangle $bbox -tags [list sel s:$node] \
 			-fill $selbg -outline $selbg]
-            $path.c itemconfigure "n:$node" -fill $selfg
+	    if {$selfg != ""} {
+		# Don't allow an empty fill - that would be transparent
+		$path.c itemconfigure "n:$node" -fill $selfg
+	    }
             $path.c lower $id
         }
     }
@@ -1526,7 +1549,7 @@ proc Tree::_redraw_idle { path level } {
     upvar 0  $path data
 
     if { [Widget::getoption $path -redraw] && $data(upd,afterid) == "" } {
-        set data(upd,afterid) [after idle Tree::_redraw_tree $path]
+        set data(upd,afterid) [after idle [list Tree::_redraw_tree $path]]
     }
     if { $level > $data(upd,level) } {
         set data(upd,level) $level
@@ -1546,7 +1569,7 @@ proc Tree::_init_drag_cmd { path X Y top } {
          [string equal $item "img"]  ||
          [string equal $item "win"] } {
         set node [Tree::_get_node_name $path current 2]
-        if { [set cmd [Widget::getoption $path -draginitcmd]] != "" } {
+        if {[llength [set cmd [Widget::getoption $path -draginitcmd]]]} {
             return [uplevel \#0 $cmd [list $path $node $top]]
         }
         if { [set type [Widget::getoption $path -dragtype]] == "" } {
@@ -1575,10 +1598,9 @@ proc Tree::_drop_cmd { path source X Y op type dnddata } {
         set data(dnd,afterid) ""
     }
     set data(dnd,scroll) ""
-    if { [llength $data(dnd,node)] } {
-        if { [set cmd [Widget::getoption $path -dropcmd]] != "" } {
-            return [uplevel \#0 $cmd [list $path $source $data(dnd,node) $op $type $dnddata]]
-        }
+    if {[llength [set cmd [Widget::getoption $path -dropcmd]]]} {
+	return [uplevel \#0 $cmd \
+		    [list $path $source $data(dnd,node) $op $type $dnddata]]
     }
     return 0
 }
@@ -1724,7 +1746,7 @@ proc Tree::_over_cmd { path source event X Y op type dnddata } {
             }
         }
 
-    if { $vmode && [set cmd [Widget::getoption $path -dropovercmd]] != "" } {
+    if {$vmode && [llength [set cmd [Widget::getoption $path -dropovercmd]]]} {
         # user-defined dropover command
         set res     [uplevel \#0 $cmd [list $path $source $target $op $type $dnddata]]
         set code    [lindex $res 0]
@@ -1823,7 +1845,7 @@ proc Tree::_auto_scroll { path x y } {
 
     set data(dnd,scroll) $scroll
     if { [string length $scroll] && ![string length $data(dnd,afterid)] } {
-        set data(dnd,afterid) [after 200 Tree::_scroll $path $scroll]
+        set data(dnd,afterid) [after 200 [list Tree::_scroll $path $scroll]]
     }
     return $data(dnd,afterid)
 }
@@ -1832,14 +1854,15 @@ proc Tree::_auto_scroll { path x y } {
 # ----------------------------------------------------------------------------
 #  Command Tree::_scroll
 # ----------------------------------------------------------------------------
-proc Tree::_scroll { path cmd dir } {
+proc Tree::_scroll { path scroll } {
     variable $path
     upvar 0  $path data
-
+    set cmd [lindex $scroll 0]
+    set dir [lindex $scroll 1]
     if { ($dir == -1 && [lindex [$path.c $cmd] 0] > 0) ||
          ($dir == 1  && [lindex [$path.c $cmd] 1] < 1) } {
         $path.c $cmd scroll $dir units
-        set data(dnd,afterid) [after 100 Tree::_scroll $path $cmd $dir]
+        set data(dnd,afterid) [after 50 [list Tree::_scroll $path $scroll]]
     } else {
         set data(dnd,afterid) ""
         DropSite::setcursor dot
@@ -1863,6 +1886,11 @@ proc Tree::_scroll { path cmd dir } {
 #	None.
 
 proc Tree::_keynav {which win} {
+    # check for an empty tree
+    if {[$win nodes root] eq ""} {
+        return
+    }
+
     # Keyboard navigation is riddled with special cases.  In order to avoid
     # the complex logic, we will instead make a list of all the visible,
     # selectable nodes, then do a simple next or previous operation.
@@ -1936,7 +1964,7 @@ proc Tree::_keynav {which win} {
                 }
             } else {
                 $win itemconfigure $node -open 1
-                if { [set cmd [Widget::getoption $win -opencmd]] != "" } {
+                if {[llength [set cmd [Widget::getoption $win -opencmd]]]} {
                     uplevel \#0 $cmd [list $node]
                 }
                 return
@@ -1951,7 +1979,7 @@ proc Tree::_keynav {which win} {
 	    set open [$win itemcget $node -open]
 	    if { $open } {
 		$win itemconfigure $node -open 0
-                if { [set cmd [Widget::getoption $win -closecmd]] != "" } {
+                if {[llength [set cmd [Widget::getoption $win -closecmd]]]} {
                     uplevel \#0 $cmd [list $node]
                 }
 		return
@@ -1988,13 +2016,13 @@ proc Tree::_keynav {which win} {
 		if {$open} {
 		    # Node was open, is now closed. Call the close-cmd
 
-		    if { [set cmd [Widget::getoption $win -closecmd]] != "" } {
+		    if {[llength [set cmd [Widget::getoption $win -closecmd]]]} {
 			uplevel \#0 $cmd [list $node]
 		    }
 		} else {
 		    # Node was closed, is now open. Call the open-cmd
 
-		    if { [set cmd [Widget::getoption $win -opencmd]] != "" } {
+		    if {[llength [set cmd [Widget::getoption $win -opencmd]]]} {
 			uplevel \#0 $cmd [list $node]
 		    }
                 }
@@ -2021,7 +2049,7 @@ proc Tree::_keynav {which win} {
 proc Tree::_get_current_node {win} {
     if {[info exists selectTree::selectCursor($win)]} {
 	set result $selectTree::selectCursor($win)
-    } elseif {[set selList [$win selection get]] != {}} {
+    } elseif {[llength [set selList [$win selection get]]]} {
 	set result [lindex $selList 0]
     } else {
 	set result ""
@@ -2064,8 +2092,12 @@ proc Tree::_set_current_node {win node} {
 # Results:
 #	node	name of the tree node.
 
-proc Tree::_get_node_name {path {item current} {tagindex end-1}} {
-    return [string range [lindex [$path.c gettags $item] $tagindex] 2 end]
+proc Tree::_get_node_name {path {item current} {tagindex end-1} {truename 0}} {
+    set node [string range [lindex [$path.c gettags $item] $tagindex] 2 end]
+    if {$truename} {
+	return [_node_name_rev $path $node]
+    }
+    return $node
 }
 
 # Tree::_get_node_padx --
@@ -2116,8 +2148,9 @@ proc Tree::_get_node_deltax {path node} {
 # Results:
 #	list		The list of tags to apply to the canvas item
 proc Tree::_get_node_tags {path node {tags ""}} {
-    eval [list lappend list TreeItemSentinal] $tags
-    if {[Widget::getoption $path.$node -helptext] == ""} { return $list }
+    eval [linsert $tags 0 lappend list TreeItemSentinal]
+    if {[Widget::getoption $path.$node -helptext] == "" &&
+        [Widget::getoption $path.$node -helpcmd]  == ""} { return $list }
 
     switch -- [Widget::getoption $path.$node -helptype] {
 	balloon {
@@ -2145,34 +2178,27 @@ proc Tree::_set_help { path node } {
     Widget::getVariable $path help
 
     set item $path.$node
-    set opts [list -helptype -helptext -helpvar]
-    foreach {cty ctx cv} [eval [list Widget::hasChangedX $item] $opts] break
+    set opts [list -helptype -helptext -helpvar -helpcmd]
+    foreach {cty ctx cv cc} [eval [linsert $opts 0 Widget::hasChangedX $item]] break
     set text [Widget::getoption $item -helptext]
+    set cmd  [Widget::getoption $item -helpcmd]
 
-    ## If we've never set help for this item before, and text is not blank,
-    ## we need to setup help.  We also need to reset help if any of the
+    ## If we've never set help for this item before, and text or cmd is not
+    ## blank, we need to setup help. We also need to reset help if any of the
     ## options have changed.
-    if { (![info exists help($node)] && $text != "") || $cty || $ctx || $cv } {
+    if { (![info exists help($node)] && ($text != "" || $cmd != ""))
+         || $cty || $ctx || $cv } {
 	set help($node) 1
 	set type [Widget::getoption $item -helptype]
-        switch $type {
-            balloon {
-		DynamicHelp::register $path.c balloon n:$node $text
-		DynamicHelp::register $path.c balloon i:$node $text
-		DynamicHelp::register $path.c balloon b:$node $text
-            }
-            variable {
-		set var [Widget::getoption $item -helpvar]
-		DynamicHelp::register $path.c variable n:$node $var $text
-		DynamicHelp::register $path.c variable i:$node $var $text
-		DynamicHelp::register $path.c variable b:$node $var $text
-            }
-        }
+        set var [Widget::getoption $item -helpvar]
+        DynamicHelp::add $path.c -item n:$node -type $type -text $text -variable $var -command $cmd
+        DynamicHelp::add $path.c -item i:$node -type $type -text $text -variable $var -command $cmd
+        DynamicHelp::add $path.c -item b:$node -type $type -text $text -variable $var -command $cmd
     }
 }
 
 proc Tree::_mouse_select { path cmd args } {
-    eval selection [list $path] [list $cmd] $args
+    eval [linsert $args 0 selection $path $cmd]
     switch -- $cmd {
         "add" - "clear" - "remove" - "set" - "toggle" {
             event generate $path <<TreeSelect>>
@@ -2180,9 +2206,15 @@ proc Tree::_mouse_select { path cmd args } {
     }
 }
 
-
 proc Tree::_node_name { path node } {
-    set map [list & _ | _ ^ _ ! _]
+    # Make sure node names are safe as tags and variable names
+    set map [list & \1 | \2 ^ \3 ! \4 :: \5]
+    return  [string map $map $node]
+}
+
+proc Tree::_node_name_rev { path node } {
+    # Allow reverse interpretation of node names
+    set map [list \1 & \2 | \3 ^ \4 ! \5 ::]
     return  [string map $map $node]
 }
 
