@@ -7,9 +7,9 @@
 
 	COPYRIGHT NOTICE, DISCLAIMER, and LICENSE:
 
-	CxImage version 5.99c 17/Oct/2004
+	CxImage version 6.0.0 02/Feb/2008
 
-	CxImage : Copyright (C) 2001 - 2004, Davide Pizzolato
+	CxImage : Copyright (C) 2001 - 2008, Davide Pizzolato
 
 	Original CImage and CImageIterator implementation are:
 	Copyright (C) 1995, Alejandro Aguilar Sierra (asierra(at)servidor(dot)unam(dot)mx)
@@ -40,8 +40,8 @@
 
   --------------------------------------------------------------------------------
 
-	Other information: about CxImage, and the latest version, can be found at the
-	CxImage home page: http://www.xdp.it
+	Other information about CxImage, and the latest version, can be found at the
+	CxImage home page: http://www.xdp.it/cximage/
 
   --------------------------------------------------------------------------------
  */
@@ -63,62 +63,71 @@
 /////////////////////////////////////////////////////////////////////////////
 // CxImage formats enumerator
 enum ENUM_CXIMAGE_FORMATS{
-CXIMAGE_FORMAT_UNKNOWN,
+CXIMAGE_FORMAT_UNKNOWN = 0,
 #if CXIMAGE_SUPPORT_BMP
-CXIMAGE_FORMAT_BMP,
+CXIMAGE_FORMAT_BMP = 1,
 #endif
 #if CXIMAGE_SUPPORT_GIF
-CXIMAGE_FORMAT_GIF,
+CXIMAGE_FORMAT_GIF = 2,
 #endif
 #if CXIMAGE_SUPPORT_JPG
-CXIMAGE_FORMAT_JPG,
+CXIMAGE_FORMAT_JPG = 3,
 #endif
 #if CXIMAGE_SUPPORT_PNG
-CXIMAGE_FORMAT_PNG,
-#endif
-#if CXIMAGE_SUPPORT_MNG
-CXIMAGE_FORMAT_MNG,
+CXIMAGE_FORMAT_PNG = 4,
 #endif
 #if CXIMAGE_SUPPORT_ICO
-CXIMAGE_FORMAT_ICO,
+CXIMAGE_FORMAT_ICO = 5,
 #endif
 #if CXIMAGE_SUPPORT_TIF
-CXIMAGE_FORMAT_TIF,
+CXIMAGE_FORMAT_TIF = 6,
 #endif
 #if CXIMAGE_SUPPORT_TGA
-CXIMAGE_FORMAT_TGA,
+CXIMAGE_FORMAT_TGA = 7,
 #endif
 #if CXIMAGE_SUPPORT_PCX
-CXIMAGE_FORMAT_PCX,
+CXIMAGE_FORMAT_PCX = 8,
 #endif
 #if CXIMAGE_SUPPORT_WBMP
-CXIMAGE_FORMAT_WBMP,
+CXIMAGE_FORMAT_WBMP = 9,
 #endif
 #if CXIMAGE_SUPPORT_WMF
-CXIMAGE_FORMAT_WMF,
-#endif
-#if CXIMAGE_SUPPORT_J2K
-CXIMAGE_FORMAT_J2K,
-#endif
-#if CXIMAGE_SUPPORT_JBG
-CXIMAGE_FORMAT_JBG,
+CXIMAGE_FORMAT_WMF = 10,
 #endif
 #if CXIMAGE_SUPPORT_JP2
-CXIMAGE_FORMAT_JP2,
+CXIMAGE_FORMAT_JP2 = 11,
 #endif
 #if CXIMAGE_SUPPORT_JPC
-CXIMAGE_FORMAT_JPC,
+CXIMAGE_FORMAT_JPC = 12,
 #endif
 #if CXIMAGE_SUPPORT_PGX
-CXIMAGE_FORMAT_PGX,
+CXIMAGE_FORMAT_PGX = 13,
 #endif
 #if CXIMAGE_SUPPORT_PNM
-CXIMAGE_FORMAT_PNM,
+CXIMAGE_FORMAT_PNM = 14,
 #endif
 #if CXIMAGE_SUPPORT_RAS
-CXIMAGE_FORMAT_RAS,
+CXIMAGE_FORMAT_RAS = 15,
 #endif
-CMAX_IMAGE_FORMATS
+#if CXIMAGE_SUPPORT_JBG
+CXIMAGE_FORMAT_JBG = 16,
+#endif
+#if CXIMAGE_SUPPORT_MNG
+CXIMAGE_FORMAT_MNG = 17,
+#endif
+#if CXIMAGE_SUPPORT_SKA
+CXIMAGE_FORMAT_SKA = 18,
+#endif
+#if CXIMAGE_SUPPORT_RAW
+CXIMAGE_FORMAT_RAW = 19,
+#endif
+CMAX_IMAGE_FORMATS = CXIMAGE_SUPPORT_BMP + CXIMAGE_SUPPORT_GIF + CXIMAGE_SUPPORT_JPG +
+					 CXIMAGE_SUPPORT_PNG + CXIMAGE_SUPPORT_MNG + CXIMAGE_SUPPORT_ICO +
+					 CXIMAGE_SUPPORT_TIF + CXIMAGE_SUPPORT_TGA + CXIMAGE_SUPPORT_PCX +
+					 CXIMAGE_SUPPORT_WBMP+ CXIMAGE_SUPPORT_WMF +
+					 CXIMAGE_SUPPORT_JBG + CXIMAGE_SUPPORT_JP2 + CXIMAGE_SUPPORT_JPC +
+					 CXIMAGE_SUPPORT_PGX + CXIMAGE_SUPPORT_PNM + CXIMAGE_SUPPORT_RAS +
+					 CXIMAGE_SUPPORT_SKA + CXIMAGE_SUPPORT_RAW + 1
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -138,8 +147,7 @@ typedef struct tagCxImageInfo {
 	long	nEscape;			///< escape
 	long	nBkgndIndex;		///< used for GIF, PNG, MNG
 	RGBQUAD nBkgndColor;		///< used for RGB transparency
-	int 	m_dispmeth;		///< used for GIF
-	BYTE	nQuality;			///< used for JPEG
+	float	fQuality;			///< used for JPEG, JPEG2000 (0.0f ... 100.0f)
 	BYTE	nJpegScale;			///< used for JPEG [ignacio]
 	long	nFrame;				///< used for TIF, GIF, MNG : actual frame
 	long	nNumFrames;			///< used for TIF, GIF, MNG : total number of frames
@@ -158,8 +166,9 @@ typedef struct tagCxImageInfo {
 	bool	last_c_isvalid;
 	long	nNumLayers;
 	DWORD	dwFlags;			///< 0x??00000 = reserved, 0x00??0000 = blend mode, 0x0000???? = layer id - user flags
+	BYTE	dispmeth;
 	bool	bGetAllFrames;
-	CxImage ** GifFrames;
+	bool	bLittleEndianHost;
 
 } CXIMAGEINFO;
 
@@ -172,10 +181,15 @@ struct rgb_color { BYTE r,g,b; };
 // members must be initialized with the InitTextInfo(&this) function.
 typedef struct tagCxTextInfo
 {
+#if defined (_WIN32_WCE)
+	TCHAR    text[256];  ///< text for windows CE
+#else
 	TCHAR    text[4096]; ///< text (char -> TCHAR for UNICODE [Cesar M])
+#endif
 	LOGFONT  lfont;      ///< font and codepage data
     COLORREF fcolor;     ///< foreground color
     long     align;      ///< DT_CENTER, DT_RIGHT, DT_LEFT aligment for multiline text
+    BYTE     smooth;     ///< text smoothing option. Default is false.
     BYTE     opaque;     ///< text has background or hasn't. Default is true.
 						 ///< data for background (ignored if .opaque==FALSE) 
     COLORREF bcolor;     ///< background color
@@ -195,20 +209,25 @@ public:
 	CxImage(FILE * stream, DWORD imagetype);
 	CxImage(CxFile * stream, DWORD imagetype);
 	CxImage(BYTE * buffer, DWORD size, DWORD imagetype);
-	virtual ~CxImage() { Destroy(); DestroyGifFrames();};
+	virtual ~CxImage() { DestroyFrames(); Destroy(); };
 	CxImage& operator = (const CxImage&);
 //@}
 
 /** \addtogroup Initialization */ //@{
 	void*	Create(DWORD dwWidth, DWORD dwHeight, DWORD wBpp, DWORD imagetype = 0);
 	bool	Destroy();
-	bool	DestroyGifFrames();
+	bool	DestroyFrames();
 	void	Clear(BYTE bval=0);
 	void	Copy(const CxImage &src, bool copypixels = true, bool copyselection = true, bool copyalpha = true);
-	bool	Transfer(CxImage &from);
+	bool	Transfer(CxImage &from, bool bTransferFrames = true);
 	bool	CreateFromArray(BYTE* pArray,DWORD dwWidth,DWORD dwHeight,DWORD dwBitsperpixel, DWORD dwBytesperline, bool bFlipImage);
 	bool	CreateFromMatrix(BYTE** ppMatrix,DWORD dwWidth,DWORD dwHeight,DWORD dwBitsperpixel, DWORD dwBytesperline, bool bFlipImage);
 	void	FreeMemory(void* memblock);
+
+	DWORD Dump(BYTE * dst);
+	DWORD UnDump(const BYTE * src);
+	DWORD DumpSize();
+
 //@}
 
 /** \addtogroup Attributes */ //@{
@@ -223,8 +242,8 @@ public:
 	WORD	GetBpp() const;
 	DWORD	GetType() const;
 	const char*	GetLastError();
-	const TCHAR* GetVersion();
-	const float GetVersionNumber();
+	static const TCHAR* GetVersion();
+	static const float GetVersionNumber();
 
 	DWORD	GetFrameDelay() const;
 	void	SetFrameDelay(DWORD d);
@@ -234,6 +253,8 @@ public:
 
 	BYTE	GetJpegQuality() const;
 	void	SetJpegQuality(BYTE q);
+	float	GetJpegQualityF() const;
+	void	SetJpegQualityF(float q);
 
 	BYTE	GetJpegScale() const;
 	void	SetJpegScale(BYTE q);
@@ -256,8 +277,6 @@ public:
 	void	SetTransIndex(long idx);
 	void	SetTransColor(RGBQUAD rgb);
 	bool	IsTransparent() const;
-	void	SetDisposalMethod(int dm);
-	long	GetDisposalMethod();
 
 	DWORD	GetCodecOption(DWORD imagetype = 0);
 	bool	SetCodecOption(DWORD opt, DWORD imagetype = 0);
@@ -265,6 +284,20 @@ public:
 	DWORD	GetFlags() const;
 	void	SetFlags(DWORD flags, bool bLockReservedFlags = true);
 
+	BYTE	GetDisposalMethod() const;
+	void	SetDisposalMethod(BYTE dm);
+
+	bool	SetType(DWORD type);
+
+	static DWORD GetNumTypes();
+	static DWORD GetTypeIdFromName(const TCHAR* ext);
+	static DWORD GetTypeIdFromIndex(const DWORD index);
+	static DWORD GetTypeIndexFromId(const DWORD id);
+
+	bool	GetRetreiveAllFrames() const;
+	void	SetRetreiveAllFrames(bool flag);
+	CxImage * GetFrame(long nFrame) const;
+
 	//void*	GetUserData() const {return info.pUserData;}
 	//void	SetUserData(void* pUserData) {info.pUserData = pUserData;}
 //@}
@@ -289,12 +322,14 @@ public:
 	void	SetPaletteColor(BYTE idx, RGBQUAD c);
 	void	SetPaletteColor(BYTE idx, COLORREF cr);
 	void	SwapIndex(BYTE idx1, BYTE idx2);
+	void	SwapRGB2BGR();
 	void	SetStdPalette();
 //@}
 
 /** \addtogroup Pixel */ //@{
 	bool	IsInside(long x, long y);
 	bool	IsTransparent(long x,long y);
+	bool	GetTransparentMask(CxImage* iDst = 0);
 	RGBQUAD GetPixelColor(long x,long y, bool bGetAlpha = true);
 	BYTE	GetPixelIndex(long x,long y);
 	BYTE	GetPixelGray(long x, long y);
@@ -309,8 +344,10 @@ public:
 protected:
 /** \addtogroup Protected */ //@{
 	BYTE BlindGetPixelIndex(const long x,const long y);
-	RGBQUAD BlindGetPixelColor(const long x,const long y);
+	RGBQUAD BlindGetPixelColor(const long x,const long y, bool bGetAlpha = true);
 	void *BlindGetPixelPointer(const long x,const  long y);
+	void BlindSetPixelColor(long x,long y,RGBQUAD c, bool bSetAlpha = false);
+	void BlindSetPixelIndex(long x,long y,BYTE i);
 //@}
 
 public:
@@ -346,7 +383,9 @@ public:
 		IM_GAUSSIAN		=13,
 		IM_QUADRATIC	=14,
 		IM_MITCHELL		=15,
-		IM_CATROM		=16
+		IM_CATROM		=16,
+		IM_HANNING		=17,
+		IM_POWER		=18
 	};
 	RGBQUAD GetPixelColorInterpolated(float x,float y, InterpolationMethod const inMethod=IM_BILINEAR, OverflowMethod const ofMethod=OM_BACKGROUND, RGBQUAD* const rplColor=0);
 	RGBQUAD GetAreaColorInterpolated(float const xc, float const yc, float const w, float const h, InterpolationMethod const inMethod, OverflowMethod const ofMethod=OM_BACKGROUND, RGBQUAD* const rplColor=0);
@@ -378,14 +417,14 @@ public:
 	static float KernelQuadratic(const float x);
 	static float KernelMitchell(const float x);
 	static float KernelCatrom(const float x);
+	static float KernelHanning(const float x);
+	static float KernelPower(const float x, const float a = 2);
 //@}
 #endif //CXIMAGE_SUPPORT_INTERPOLATION
 	
 /** \addtogroup Painting */ //@{
-#if CXIMAGE_SUPPORT_WINCE
-	long	Blt(HDC pDC, long x=0, long y=0);
-#endif
 #if CXIMAGE_SUPPORT_WINDOWS
+	long	Blt(HDC pDC, long x=0, long y=0);
 	HBITMAP MakeBitmap(HDC hdc = NULL);
 	HANDLE	CopyToHandle();
 	bool	CreateFromHANDLE(HANDLE hMem);		//Windows objects (clipboard)
@@ -419,6 +458,9 @@ public:
 	bool Decode(FILE * hFile, DWORD imagetype);
 	bool Decode(CxFile * hFile, DWORD imagetype);
 	bool Decode(BYTE * buffer, DWORD size, DWORD imagetype);
+
+	bool CheckFormat(CxFile * hFile, DWORD imagetype = 0);
+	bool CheckFormat(BYTE * buffer, DWORD size, DWORD imagetype = 0);
 //@}
 #endif //CXIMAGE_SUPPORT_DECODE
 
@@ -442,8 +484,8 @@ public:
 	bool Encode(FILE *hFile, CxImage ** pImages, int pagecount, DWORD imagetype);
 	bool Encode(BYTE * &buffer, long &size, DWORD imagetype);
 
-	bool Encode2RGBA(CxFile *hFile);
-	bool Encode2RGBA(BYTE * &buffer, long &size);
+	bool Encode2RGBA(CxFile *hFile, bool bFlipY = false);
+	bool Encode2RGBA(BYTE * &buffer, long &size, bool bFlipY = false);
 //@}
 #endif //CXIMAGE_SUPPORT_ENCODE
 
@@ -457,17 +499,13 @@ public:
 	long GetNumFrames() const;
 	long GetFrame() const;
 	void SetFrame(long nFrame);
-		
-	CxImage * GetFrameNo(long nFrame) const;
-	void RetreiveAllFrame();
-	void RetreiveSingleFrame();
 //@}
 
 #if CXIMAGE_SUPPORT_BASICTRANSFORMATIONS
 /** \addtogroup BasicTransformations */ //@{
 	bool GrayScale();
-	bool Flip();
-	bool Mirror();
+	bool Flip(bool bFlipSelection = false, bool bFlipAlpha = true);
+	bool Mirror(bool bMirrorSelection = false, bool bMirrorAlpha = true);
 	bool Negative();
 	bool RotateLeft(CxImage* iDst = NULL);
 	bool RotateRight(CxImage* iDst = NULL);
@@ -497,15 +535,16 @@ public:
 	bool Expand(long newx, long newy, RGBQUAD canvascolor, CxImage* iDst = 0);
 	bool Thumbnail(long newx, long newy, RGBQUAD canvascolor, CxImage* iDst = 0);
 	bool CircleTransform(int type,long rmax=0,float Koeff=1.0f);
-	bool RedEyeRemove();
-	bool QIShrink(long newx, long newy, CxImage* const iDst = NULL);
+	bool RedEyeRemove(float strength = 0.8f);
+	bool QIShrink(long newx, long newy, CxImage* const iDst = NULL, bool bChangeBpp = false);
+
 //@}
 #endif //CXIMAGE_SUPPORT_TRANSFORMATION
 
 #if CXIMAGE_SUPPORT_DSP
 /** \addtogroup DSP */ //@{
 	bool Contour();
-	bool HistogramStretch(long method = 0);
+	bool HistogramStretch(long method = 0, double threshold = 0);
 	bool HistogramEqualize();
 	bool HistogramNormalize();
 	bool HistogramRoot();
@@ -518,8 +557,11 @@ public:
 	bool Noise(long level);
 	bool Median(long Ksize=3);
 	bool Gamma(float gamma);
+	bool GammaRGB(float gammaR, float gammaG, float gammaB);
 	bool ShiftRGB(long r, long g, long b);
 	bool Threshold(BYTE level);
+	bool Threshold(CxImage* pThresholdMask);
+	bool Threshold2(BYTE level, bool bDirection, RGBQUAD nBkgndColor, bool bSetAlpha = false);
 	bool Colorize(BYTE hue, BYTE sat, float blend = 1.0f);
 	bool Light(long brightness, long contrast = 0);
 	float Mean();
@@ -528,12 +570,23 @@ public:
 	bool Dilate(long Ksize=2);
 	bool Edge(long Ksize=2);
 	void HuePalette(float correction=1);
-	enum ImageOpType { OpAdd, OpAnd, OpXor, OpOr, OpMask, OpSrcCopy, OpDstCopy, OpSub, OpSrcBlend, OpScreen };
+	enum ImageOpType { OpAdd, OpAnd, OpXor, OpOr, OpMask, OpSrcCopy, OpDstCopy, OpSub, OpSrcBlend, OpScreen, OpAvg };
 	void Mix(CxImage & imgsrc2, ImageOpType op, long lXOffset = 0, long lYOffset = 0, bool bMixAlpha = false);
 	void MixFrom(CxImage & imagesrc2, long lXOffset, long lYOffset);
-	bool UnsharpMask(float radius = 5.0, float amount = 0.5, int threshold = 0);
+	bool UnsharpMask(float radius = 5.0f, float amount = 0.5f, int threshold = 0);
 	bool Lut(BYTE* pLut);
 	bool Lut(BYTE* pLutR, BYTE* pLutG, BYTE* pLutB, BYTE* pLutA = 0);
+	bool GaussianBlur(float radius = 1.0f, CxImage* iDst = 0);
+	bool TextBlur(BYTE threshold = 100, BYTE decay = 2, BYTE max_depth = 5, bool bBlurHorizontal = true, bool bBlurVertical = true, CxImage* iDst = 0);
+	bool SelectiveBlur(float radius = 1.0f, BYTE threshold = 25, CxImage* iDst = 0);
+	bool Solarize(BYTE level = 128, bool bLinkedChannels = true);
+	bool FloodFill(const long xStart, const long yStart, const RGBQUAD cFillColor, const BYTE tolerance = 0,
+					BYTE nOpacity = 255, const bool bSelectFilledArea = false, const BYTE nSelectionLevel = 255);
+	bool Saturate(const long saturation, const long colorspace = 1);
+	bool ConvertColorSpace(const long dstColorSpace, const long srcColorSpace);
+	int  OptimalThreshold(long method = 0, RECT * pBox = 0, CxImage* pContrastMask = 0);
+	bool AdaptiveThreshold(long method = 0, long nBoxSize = 64, CxImage* pContrastMask = 0, long nBias = 0, float fGlobalLocalBalance = 0.5f);
+
 //@}
 
 protected:
@@ -546,6 +599,7 @@ protected:
 	int gen_convolve_matrix (float radius, float **cmatrix_p);
 	float* gen_lookup_table (float *cmatrix, int cmatrix_length);
 	void blur_line (float *ctable, float *cmatrix, int cmatrix_length, BYTE* cur_col, BYTE* dest_col, int y, long bytes);
+	void blur_text (BYTE threshold, BYTE decay, BYTE max_depth, CxImage* iSrc, CxImage* iDst, BYTE bytes);
 //@}
 
 public:
@@ -572,28 +626,44 @@ public:
 
 #if CXIMAGE_SUPPORT_SELECTION
 /** \addtogroup Selection */ //@{
-	bool SelectionClear();
+	bool SelectionClear(BYTE level = 0);
 	bool SelectionCreate();
 	bool SelectionDelete();
 	bool SelectionInvert();
-	bool SelectionAddRect(RECT r);
-	bool SelectionAddEllipse(RECT r);
-	bool SelectionAddPolygon(POINT *points, long npoints);
-	bool SelectionAddColor(RGBQUAD c);
-	bool SelectionAddPixel(int x, int y);
+	bool SelectionMirror();
+	bool SelectionFlip();
+	bool SelectionAddRect(RECT r, BYTE level = 255);
+	bool SelectionAddEllipse(RECT r, BYTE level = 255);
+	bool SelectionAddPolygon(POINT *points, long npoints, BYTE level = 255);
+	bool SelectionAddColor(RGBQUAD c, BYTE level = 255);
+	bool SelectionAddPixel(long x, long y, BYTE level = 255);
 	bool SelectionCopy(CxImage &from);
 	bool SelectionIsInside(long x, long y);
 	bool SelectionIsValid();
 	void SelectionGetBox(RECT& r);
 	bool SelectionToHRGN(HRGN& region);
 	bool SelectionSplit(CxImage *dest);
+	BYTE SelectionGet(const long x,const long y);
+	bool SelectionSet(CxImage &from);
+	void SelectionRebuildBox();
+	BYTE* SelectionGetPointer(const long x = 0,const long y = 0);
 //@}
+
+protected:
+/** \addtogroup Protected */ //@{
+	bool BlindSelectionIsInside(long x, long y);
+	BYTE BlindSelectionGet(const long x,const long y);
+	void SelectionSet(const long x,const long y,const BYTE level);
+//@}
+
+public:
+
 #endif //CXIMAGE_SUPPORT_SELECTION
 
 #if CXIMAGE_SUPPORT_ALPHA
 /** \addtogroup Alpha */ //@{
 	void AlphaClear();
-	void AlphaCreate();
+	bool AlphaCreate();
 	void AlphaDelete();
 	void AlphaInvert();
 	bool AlphaMirror();
@@ -609,6 +679,7 @@ public:
 	void AlphaSetMax(BYTE nAlphaMax);
 	bool AlphaIsValid();
 	BYTE* AlphaGetPointer(const long x = 0,const long y = 0);
+	bool AlphaFromTransparency();
 
 	void AlphaPaletteClear();
 	void AlphaPaletteEnable(bool enable=true);
@@ -632,6 +703,8 @@ public:
 	CxImage* GetLayer(long position);
 	CxImage* GetParent() const;
 	long GetNumLayers() const;
+	long LayerDrawAll(HDC hdc, long x=0, long y=0, long cx = -1, long cy = -1, RECT* pClipRect = 0, bool bSmooth = false);
+	long LayerDrawAll(HDC hdc, const RECT& rect, RECT* pClipRect=NULL, bool bSmooth = false);
 //@}
 #endif //CXIMAGE_SUPPORT_LAYERS
 
@@ -639,18 +712,22 @@ protected:
 /** \addtogroup Protected */ //@{
 	void Startup(DWORD imagetype = 0);
 	void CopyInfo(const CxImage &src);
-	void Ghost(CxImage *src);
+	void Ghost(const CxImage *src);
 	void RGBtoBGR(BYTE *buffer, int length);
 	static float HueToRGB(float n1,float n2, float hue);
-	void Bitfield2RGB(BYTE *src, WORD redmask, WORD greenmask, WORD bluemask, BYTE bpp);
+	void Bitfield2RGB(BYTE *src, DWORD redmask, DWORD greenmask, DWORD bluemask, BYTE bpp);
 	static int CompareColors(const void *elem1, const void *elem2);
+	short xima_ntohs(const short word);
+	long xima_ntohl(const long dword);
+	void bihtoh(BITMAPINFOHEADER* bih);
 
 	void*				pDib; //contains the header, the palette, the pixels
     BITMAPINFOHEADER    head; //standard header
 	CXIMAGEINFO			info; //extended information
 	BYTE*				pSelection;	//selected region
 	BYTE*				pAlpha; //alpha channel
-	CxImage**			pLayers; //generic layers
+	CxImage**			ppLayers; //generic layers
+	CxImage**			ppFrames;
 //@}
 };
 
