@@ -1,6 +1,6 @@
 // xImaPal.cpp : Palette and Pixel functions
 /* 07/08/2001 v1.00 - Davide Pizzolato - www.xdp.it
- * CxImage version 5.99c 17/Oct/2004
+ * CxImage version 6.0.0 02/Feb/2008
  */
 
 #include "ximage.h"
@@ -83,8 +83,8 @@ RGBQUAD CxImage::GetPaletteColor(BYTE idx)
 			rgb.rgbGreen=iDst[ldx++];
 			rgb.rgbRed =iDst[ldx++];
 			rgb.rgbReserved = iDst[ldx];
-			if ( IsTransparent() )
-				rgb.rgbReserved = (idx==GetTransIndex()) ? 0 : 255;
+            if ( IsTransparent() )
+                rgb.rgbReserved = (idx==GetTransIndex()) ? 0 : 255;
 		}
 	}
 	return rgb;
@@ -98,7 +98,7 @@ BYTE CxImage::GetPixelIndex(long x,long y)
 	if ((pDib==NULL)||(head.biClrUsed==0)) return 0;
 
 	if ((x<0)||(y<0)||(x>=head.biWidth)||(y>=head.biHeight)) {
-		if (info.nBkgndIndex != -1)	return (BYTE)info.nBkgndIndex;
+		if (info.nBkgndIndex >= 0)	return (BYTE)info.nBkgndIndex;
 		else return *info.pImage;
 	}
 	if (head.biBitCount==8){
@@ -122,7 +122,12 @@ BYTE CxImage::GetPixelIndex(long x,long y)
 BYTE CxImage::BlindGetPixelIndex(const long x,const long y)
 {
 #ifdef _DEBUG
-	if ((pDib==NULL) || (head.biClrUsed==0) || !IsInside(x,y)) throw 0;
+	if ((pDib==NULL) || (head.biClrUsed==0) || !IsInside(x,y))
+  #if CXIMAGE_SUPPORT_EXCEPTION_HANDLING
+		throw 0;
+  #else
+		return 0;
+  #endif
 #endif
 
 	if (head.biBitCount==8){
@@ -149,7 +154,7 @@ RGBQUAD CxImage::GetPixelColor(long x,long y, bool bGetAlpha)
 	RGBQUAD rgb=info.nBkgndColor; //<mpwolski>
 	if ((pDib==NULL)||(x<0)||(y<0)||
 		(x>=head.biWidth)||(y>=head.biHeight)){
-		if (info.nBkgndIndex != -1){
+		if (info.nBkgndIndex >= 0){
 			if (head.biBitCount<24) return GetPaletteColor((BYTE)info.nBkgndIndex);
 			else return info.nBkgndColor;
 		} else if (pDib) return GetPixelColor(0,0);
@@ -157,7 +162,7 @@ RGBQUAD CxImage::GetPixelColor(long x,long y, bool bGetAlpha)
 	}
 
 	if (head.biClrUsed){
-		rgb = GetPaletteColor(GetPixelIndex(x,y));
+		rgb = GetPaletteColor(BlindGetPixelIndex(x,y));
 	} else {
 		BYTE* iDst  = info.pImage + y*info.dwEffWidth + x*3;
 		rgb.rgbBlue = *iDst++;
@@ -165,7 +170,8 @@ RGBQUAD CxImage::GetPixelColor(long x,long y, bool bGetAlpha)
 		rgb.rgbRed  = *iDst;
 	}
 #if CXIMAGE_SUPPORT_ALPHA
-	if (pAlpha && bGetAlpha) rgb.rgbReserved = AlphaGet(x,y);
+	if (pAlpha && bGetAlpha)
+        rgb.rgbReserved = BlindAlphaGet(x,y);
 #else
 	rgb.rgbReserved = 0;
 #endif //CXIMAGE_SUPPORT_ALPHA
@@ -180,26 +186,30 @@ RGBQUAD CxImage::GetPixelColor(long x,long y, bool bGetAlpha)
  * In DEBUG mode an exception will be thrown, and data will be violated in non-DEBUG mode. 
  * \author ***bd*** 2.2004
  */
-RGBQUAD CxImage::BlindGetPixelColor(const long x,const long y)
+RGBQUAD CxImage::BlindGetPixelColor(const long x,const long y, bool bGetAlpha)
 {
   RGBQUAD rgb;
 #ifdef _DEBUG
-	if ((pDib==NULL) || !IsInside(x,y)) throw 0;
+	if ((pDib==NULL) || !IsInside(x,y))
+  #if CXIMAGE_SUPPORT_EXCEPTION_HANDLING
+		throw 0;
+  #else
+		{rgb.rgbReserved = 0; return rgb;}
+  #endif
 #endif
 
 	if (head.biClrUsed){
-		return GetPaletteColor(BlindGetPixelIndex(x,y));
+		rgb = GetPaletteColor(BlindGetPixelIndex(x,y));
+	} else {
+		BYTE* iDst  = info.pImage + y*info.dwEffWidth + x*3;
+		rgb.rgbBlue = *iDst++;
+		rgb.rgbGreen= *iDst++;
+		rgb.rgbRed  = *iDst;
+		rgb.rgbReserved = 0; //needed for images without alpha layer
 	}
-
-	BYTE* iDst  = info.pImage + y*info.dwEffWidth + x*3;
-	rgb.rgbBlue = *iDst++;
-	rgb.rgbGreen= *iDst++;
-	rgb.rgbRed  = *iDst;
 #if CXIMAGE_SUPPORT_ALPHA
-	if (pAlpha) 
-		rgb.rgbReserved = pAlpha[x+y*head.biWidth];
-	else
-		rgb.rgbReserved = 0;
+	if (pAlpha && bGetAlpha)
+        rgb.rgbReserved = BlindAlphaGet(x,y);
 #else
 	rgb.rgbReserved = 0;
 #endif //CXIMAGE_SUPPORT_ALPHA
@@ -212,6 +222,38 @@ BYTE CxImage::GetPixelGray(long x, long y)
 	return (BYTE)RGB2GRAY(color.rgbRed,color.rgbGreen,color.rgbBlue);
 }
 ////////////////////////////////////////////////////////////////////////////////
+void CxImage::BlindSetPixelIndex(long x,long y,BYTE i)
+{
+#ifdef _DEBUG
+	if ((pDib==NULL)||(head.biClrUsed==0)||
+		(x<0)||(y<0)||(x>=head.biWidth)||(y>=head.biHeight))
+  #if CXIMAGE_SUPPORT_EXCEPTION_HANDLING
+		throw 0;
+  #else
+		return;
+  #endif
+#endif
+
+	if (head.biBitCount==8){
+		info.pImage[y*info.dwEffWidth + x]=i;
+		return;
+	} else {
+		BYTE pos;
+		BYTE* iDst= info.pImage + y*info.dwEffWidth + (x*head.biBitCount >> 3);
+		if (head.biBitCount==4){
+			pos = (BYTE)(4*(1-x%2));
+			*iDst &= ~(0x0F<<pos);
+			*iDst |= ((i & 0x0F)<<pos);
+			return;
+		} else if (head.biBitCount==1){
+			pos = (BYTE)(7-x%8);
+			*iDst &= ~(0x01<<pos);
+			*iDst |= ((i & 0x01)<<pos);
+			return;
+		}
+	}
+}
+////////////////////////////////////////////////////////////////////////////////
 void CxImage::SetPixelIndex(long x,long y,BYTE i)
 {
 	if ((pDib==NULL)||(head.biClrUsed==0)||
@@ -242,21 +284,45 @@ void CxImage::SetPixelColor(long x,long y,COLORREF cr)
 	SetPixelColor(x,y,RGBtoRGBQUAD(cr));
 }
 ////////////////////////////////////////////////////////////////////////////////
-void CxImage::SetPixelColor(long x,long y,RGBQUAD c, bool bSetAlpha)
+void CxImage::BlindSetPixelColor(long x,long y,RGBQUAD c, bool bSetAlpha)
 {
+#ifdef _DEBUG
 	if ((pDib==NULL)||(x<0)||(y<0)||
-		(x>=head.biWidth)||(y>=head.biHeight)) return;
+		(x>=head.biWidth)||(y>=head.biHeight))
+  #if CXIMAGE_SUPPORT_EXCEPTION_HANDLING
+		throw 0;
+  #else
+		return;
+  #endif
+#endif
 	if (head.biClrUsed)
-		SetPixelIndex(x,y,GetNearestIndex(c));
+		BlindSetPixelIndex(x,y,GetNearestIndex(c));
 	else {
 		BYTE* iDst = info.pImage + y*info.dwEffWidth + x*3;
 		*iDst++ = c.rgbBlue;
 		*iDst++ = c.rgbGreen;
 		*iDst   = c.rgbRed;
-#if CXIMAGE_SUPPORT_ALPHA
-		if (bSetAlpha) AlphaSet(x,y,c.rgbReserved);
-#endif //CXIMAGE_SUPPORT_ALPHA
 	}
+#if CXIMAGE_SUPPORT_ALPHA
+	if (bSetAlpha) AlphaSet(x,y,c.rgbReserved);
+#endif //CXIMAGE_SUPPORT_ALPHA
+}
+////////////////////////////////////////////////////////////////////////////////
+void CxImage::SetPixelColor(long x,long y,RGBQUAD c, bool bSetAlpha)
+{
+	if ((pDib==NULL)||(x<0)||(y<0)||
+		(x>=head.biWidth)||(y>=head.biHeight)) return;
+	if (head.biClrUsed)
+		BlindSetPixelIndex(x,y,GetNearestIndex(c));
+	else {
+		BYTE* iDst = info.pImage + y*info.dwEffWidth + x*3;
+		*iDst++ = c.rgbBlue;
+		*iDst++ = c.rgbGreen;
+		*iDst   = c.rgbRed;
+	}
+#if CXIMAGE_SUPPORT_ALPHA
+	if (bSetAlpha) AlphaSet(x,y,c.rgbReserved);
+#endif //CXIMAGE_SUPPORT_ALPHA
 }
 ////////////////////////////////////////////////////////////////////////////////
 /**
@@ -280,7 +346,7 @@ void CxImage::BlendPixelColor(long x,long y,RGBQUAD c, float blend, bool bSetAlp
 	c.rgbGreen  = (BYTE)((c.rgbGreen * a0 + c0.rgbGreen * a1)>>8);
 
 	if (head.biClrUsed)
-		SetPixelIndex(x,y,GetNearestIndex(c));
+		BlindSetPixelIndex(x,y,GetNearestIndex(c));
 	else {
 		BYTE* iDst = info.pImage + y*info.dwEffWidth + x*3;
 		*iDst++ = c.rgbBlue;
@@ -323,7 +389,7 @@ BYTE CxImage::GetNearestIndex(RGBQUAD c)
 			j=i;
 		}
 	}
-	info.last_c_index = j;
+	info.last_c_index = (BYTE)j;
 	return (BYTE)j;
 }
 ////////////////////////////////////////////////////////////////////////////////
@@ -337,6 +403,7 @@ void CxImage::RGBtoBGR(BYTE *buffer, int length)
 	if (buffer && (head.biClrUsed==0)){
 		BYTE temp;
 		length = min(length,(int)info.dwEffWidth);
+		length = min(length,(int)(3*head.biWidth));
 		for (int i=0;i<length;i+=3){
 			temp = buffer[i]; buffer[i] = buffer[i+2]; buffer[i+2] = temp;
 		}
@@ -471,10 +538,31 @@ void CxImage::SwapIndex(BYTE idx1, BYTE idx2)
 	//swap the pixels
 	BYTE idx;
 	for(long y=0; y < head.biHeight; y++){
-		for(long x=0; x <= head.biWidth; x++){
-			idx=GetPixelIndex(x,y);
-			if (idx==idx1) SetPixelIndex(x,y,idx2);
-			if (idx==idx2) SetPixelIndex(x,y,idx1);
+		for(long x=0; x < head.biWidth; x++){
+			idx=BlindGetPixelIndex(x,y);
+			if (idx==idx1) BlindSetPixelIndex(x,y,idx2);
+			if (idx==idx2) BlindSetPixelIndex(x,y,idx1);
+		}
+	}
+}
+////////////////////////////////////////////////////////////////////////////////
+/**
+ * swap Red and Blue colors
+ */
+void CxImage::SwapRGB2BGR()
+{
+	if (!pDib) return;
+
+	if (head.biClrUsed){
+		RGBQUAD* ppal=GetPalette();
+		BYTE b;
+		if(!ppal) return;
+		for(WORD a=0;a<head.biClrUsed;a++){
+			b=ppal[a].rgbBlue; ppal[a].rgbBlue=ppal[a].rgbRed; ppal[a].rgbRed=b;
+		}
+	} else {
+		for(long y=0;y<head.biHeight;y++){
+			RGBtoBGR(GetBits(y),3*head.biWidth);
 		}
 	}
 }
@@ -500,6 +588,29 @@ bool CxImage::IsTransparent(long x, long y)
 	return false;
 }
 ////////////////////////////////////////////////////////////////////////////////
+bool CxImage::GetTransparentMask(CxImage* iDst)
+{
+	if (!pDib) return false;
+
+	CxImage tmp;
+	tmp.Create(head.biWidth, head.biHeight, 1, GetType());
+	tmp.SetStdPalette();
+	tmp.Clear(0);
+
+	for(long y=0; y<head.biHeight; y++){
+		for(long x=0; x<head.biWidth; x++){
+			if (IsTransparent(x,y)){
+				tmp.BlindSetPixelIndex(x,y,1);
+			}
+		}
+	}
+
+	if (iDst) iDst->Transfer(tmp);
+	else Transfer(tmp);
+
+	return true;
+}
+////////////////////////////////////////////////////////////////////////////////
 /**
  * Checks if image has the same palette, if any.
  * \param img = image to compare.
@@ -570,6 +681,14 @@ void CxImage::SetClrImportant(DWORD ncolors)
  */
 void* CxImage::BlindGetPixelPointer(const long x, const long y)
 {
+#ifdef _DEBUG
+	if ((pDib==NULL) || !IsInside(x,y))
+  #if CXIMAGE_SUPPORT_EXCEPTION_HANDLING
+		throw 0;
+  #else
+		return 0;
+  #endif
+#endif
   if (!IsIndexed())
     return info.pImage + y*info.dwEffWidth + x*3;
   else
@@ -655,7 +774,7 @@ void CxImage::DrawLine(int StartX, int EndX, int StartY, int EndY, RGBQUAD color
 }
 ////////////////////////////////////////////////////////////////////////////////
 /**
- * Sets a palette with standard colors for 4 and 8 bpp images.
+ * Sets a palette with standard colors for 1, 4 and 8 bpp images.
  */
 void CxImage::SetStdPalette()
 {
@@ -706,7 +825,14 @@ void CxImage::SetStdPalette()
 			memcpy(GetPalette(),pal16,64);
 			break;
 		}
+	case 1:
+		{
+			const BYTE pal2[8]={0,0,0,0,255,255,255,0};
+			memcpy(GetPalette(),pal2,8);
+			break;
+		}
 	}
+	info.last_c_isvalid = false;
 	return;
 }
 ////////////////////////////////////////////////////////////////////////////////
