@@ -55,6 +55,13 @@ extern int  ng_mpeg_apid;
 #define GETELEM(array,index,default) \
 	(index < sizeof(array)/sizeof(array[0]) ? array[index] : default)
 
+#ifdef __sun
+#include <sys/mkdev.h>		/* For major() */
+#define __u32 uint32_t
+#define __u16 uint16_t
+#define __u8  uint8_t
+#endif
+
 /* --------------------------------------------------------------------- */
 /* defines                                                               */
 
@@ -78,7 +85,9 @@ extern int  ng_mpeg_apid;
 #define VIDEO_JPEG	    17  /* JPEG (JFIF) */
 #define VIDEO_UYVY	    18  /* 4:2:2 */
 #define VIDEO_MPEG	    19  /* MPEG1/2 */
-#define VIDEO_FMT_COUNT	    20
+#define VIDEO_BAYER			20
+#define VIDEO_S910			21
+#define VIDEO_FMT_COUNT	    22
 
 #define AUDIO_NONE           0
 #define AUDIO_U8_MONO        1
@@ -605,7 +614,7 @@ int ng_dev_open(struct ng_devstate *dev);
 int ng_dev_close(struct ng_devstate *dev);
 int ng_dev_users(struct ng_devstate *dev);
 
-int ng_chardev_open(char *device, int flags, int major, int complain);
+int ng_chardev_open(char *device, int flags, int major, int complain, int is_v4l2);
 
 struct ng_reader* ng_find_reader_magic(char *filename);
 struct ng_reader* ng_find_reader_name(char *name);
@@ -647,9 +656,8 @@ void ng_yuv422p_to_lut2(void *h, struct ng_video_buf *out,
 void ng_yuv422p_to_lut4(void *h, struct ng_video_buf *out,
 			struct ng_video_buf *in);
 
-void __init yuv2rgb_init(void);
-void __init packed_init(void);
-
+void yuv2rgb_init(void);
+void packed_init(void);
 
 /* color_common.c stuff */
 void* ng_packed_init(struct ng_video_fmt *out, void *priv);
